@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.cpp,v 1.12 2003/12/09 22:23:06 marco Exp $
+ *  $Id: EphyBrowser.cpp,v 1.15 2004/01/10 21:33:04 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -74,6 +74,9 @@
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMHTMLCollection.h"
 #include "nsIDOMHTMLElement.h"
+#include "nsIDOMHTMLFormElement.h"
+#include "nsIDOMHTMLInputElement.h"
+#include "nsIDOMHTMLTextAreaElement.h"
 #include "nsIDeviceContext.h"
 #include "nsIPresContext.h"
 #include "nsIAtom.h"
@@ -113,7 +116,7 @@ EphyFaviconEventListener::HandleFaviconLink (nsIDOMNode *node)
 	NS_NAMED_LITERAL_STRING(attr_rel, "rel");
 	nsAutoString value;
 	result = linkElement->GetAttribute (attr_rel, value);
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 
 	if (value.Equals(NS_LITERAL_STRING("SHORTCUT ICON"),
 			 nsCaseInsensitiveStringComparator()) ||
@@ -123,25 +126,26 @@ EphyFaviconEventListener::HandleFaviconLink (nsIDOMNode *node)
 		NS_NAMED_LITERAL_STRING(attr_href, "href");
 		nsAutoString value;
 		result = linkElement->GetAttribute (attr_href, value);
-		if (NS_FAILED (result) || value.IsEmpty())
-			return NS_ERROR_FAILURE;
+		if (NS_FAILED (result) || value.IsEmpty()) return NS_ERROR_FAILURE;
 
 		nsCOMPtr<nsIDOMDocument> domDoc;
-		result = node->GetOwnerDocument(getter_AddRefs(domDoc));
-		if (NS_FAILED(result) || !domDoc) return NS_ERROR_FAILURE;
+		node->GetOwnerDocument(getter_AddRefs(domDoc));
+		NS_ENSURE_TRUE (domDoc, NS_ERROR_FAILURE);
 
 		nsCOMPtr<nsIDocument> doc = do_QueryInterface (domDoc);
-		if(!doc) return NS_ERROR_FAILURE;
+		NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 11
+#if MOZILLA_SNAPSHOT > 13
+		nsIURI *uri;
+		uri = doc->GetDocumentURI ();
+#elif MOZILLA_SNAPSHOT > 11
 		nsIURI *uri;
 		uri = doc->GetDocumentURL ();
-		if (uri == NULL) return NS_ERROR_FAILURE;
 #else
 		nsCOMPtr<nsIURI> uri;
-		result = doc->GetDocumentURL(getter_AddRefs(uri));
-		if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+		doc->GetDocumentURL(getter_AddRefs(uri));
 #endif
+		if (!uri) return NS_ERROR_FAILURE;
 
 		const nsACString &link = NS_ConvertUCS2toUTF8(value);
 		nsCAutoString favicon_url;
@@ -163,9 +167,8 @@ EphyFaviconEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
 
 	aDOMEvent->GetTarget(getter_AddRefs(eventTarget));
 
-	nsresult result;
-	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(eventTarget, &result);
-	if (NS_FAILED(result) || !node) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(eventTarget);
+	NS_ENSURE_TRUE (node, NS_ERROR_FAILURE);
 
 	HandleFaviconLink (node);
 
@@ -175,13 +178,8 @@ EphyFaviconEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
 NS_IMETHODIMP
 EphyPopupEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
 {
-	nsresult rv;
-
 	nsCOMPtr<nsIDOMPopupBlockedEvent> popupEvent =
-		do_QueryInterface(aDOMEvent, &rv);
-
-	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
+		do_QueryInterface(aDOMEvent);
 	if (popupEvent)
 	{
 		g_signal_emit_by_name (mOwner, "ge_popup_blocked");
@@ -190,7 +188,6 @@ EphyPopupEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
 	return NS_OK;
 }
 
-
 EphyBrowser::EphyBrowser ()
 {
 	mFaviconEventListener = nsnull;
@@ -208,26 +205,30 @@ nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 
 	gtk_moz_embed_get_nsIWebBrowser (mozembed,
 					 getter_AddRefs(mWebBrowser));
-	if (!mWebBrowser) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	mWebBrowser->GetContentDOMWindow (getter_AddRefs (mDOMWindow));
+	NS_ENSURE_TRUE (mDOMWindow, NS_ERROR_FAILURE);
 
 	/* This will instantiate an about:blank doc if necessary */
 	nsCOMPtr<nsIDOMDocument> domDocument;
 	rv = mDOMWindow->GetDocument (getter_AddRefs (domDocument));
-	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	mFaviconEventListener = new EphyFaviconEventListener();
+	if (!mFaviconEventListener) return NS_ERROR_OUT_OF_MEMORY;
+
 	mPopupEventListener = new EphyPopupEventListener();
+	if (!mPopupEventListener) return NS_ERROR_OUT_OF_MEMORY;
 
 	rv = mFaviconEventListener->Init (EPHY_EMBED (mozembed));
-	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	rv = mPopupEventListener->Init (EPHY_EMBED (mozembed));
-	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
  	rv = GetListener();
-	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	return AttachListeners();
 }
@@ -244,13 +245,13 @@ EphyBrowser::GetListener (void)
         domWindow = do_QueryInterface(domWindowExternal);
 	
 	nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(domWindow));
-  	if (!piWin) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (piWin, NS_ERROR_FAILURE);
 
   	nsCOMPtr<nsIChromeEventHandler> chromeHandler;
   	piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
 
   	mEventReceiver = do_QueryInterface(chromeHandler);
-	if (!mEventReceiver) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (mEventReceiver, NS_ERROR_FAILURE);
 
 	return NS_OK;
 }
@@ -260,17 +261,20 @@ EphyBrowser::AttachListeners(void)
 {
 	nsresult rv;
 
-  	if (!mEventReceiver) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (mEventReceiver, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIDOMEventTarget> target;
 	target = do_QueryInterface (mEventReceiver);
 
 	rv = target->AddEventListener(NS_LITERAL_STRING("DOMLinkAdded"),
 				      mFaviconEventListener, PR_FALSE);
-	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-	return target->AddEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
-					mPopupEventListener, PR_FALSE);
+	rv = target->AddEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
+				      mPopupEventListener, PR_FALSE);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	return NS_OK;
 }
 
 nsresult
@@ -278,27 +282,30 @@ EphyBrowser::DetachListeners(void)
 {
 	nsresult rv;
 
-	if (!mEventReceiver) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (mEventReceiver, NS_ERROR_FAILURE);
 	
 	nsCOMPtr<nsIDOMEventTarget> target;
 	target = do_QueryInterface (mEventReceiver);
 
 	rv = target->RemoveEventListener(NS_LITERAL_STRING("DOMLinkAdded"),
 					 mFaviconEventListener, PR_FALSE);
-	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-	return target->RemoveEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
-					   mPopupEventListener, PR_FALSE);
+	rv = target->RemoveEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
+					 mPopupEventListener, PR_FALSE);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	return NS_OK;
 }
 
 nsresult EphyBrowser::Print (nsIPrintSettings *options, PRBool preview)
 {
 	nsresult result;
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &result));
-	if (NS_FAILED(result) || !print) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	NS_ENSURE_TRUE (print, NS_ERROR_FAILURE);
 
 	if (!preview)
 	{
@@ -317,13 +324,15 @@ nsresult EphyBrowser::PrintPreviewClose (void)
 	nsresult rv;
 	PRBool isPreview = PR_FALSE;
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &rv));
-	if (NS_FAILED(rv) || !print) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	NS_ENSURE_TRUE (print, NS_ERROR_FAILURE);
 
 	rv = print->GetDoingPrintPreview(&isPreview);
-	if (NS_SUCCEEDED (rv) && isPreview == PR_TRUE)
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	if (isPreview == PR_TRUE)
 	{
 		rv = print->ExitPrintPreview();
 	}
@@ -333,38 +342,30 @@ nsresult EphyBrowser::PrintPreviewClose (void)
 
 nsresult EphyBrowser::PrintPreviewNumPages (int *numPages)
 {
-	nsresult rv;
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	NS_ENSURE_TRUE (print, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &rv));
-	if (NS_FAILED(rv) || !print) return NS_ERROR_FAILURE;
-
-	rv = print->GetPrintPreviewNumPages(numPages);
-	return rv;
+	return print->GetPrintPreviewNumPages(numPages);
 }
 
 nsresult EphyBrowser::PrintPreviewNavigate(PRInt16 navType, PRInt32 pageNum)
 {
-	nsresult rv;
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	NS_ENSURE_TRUE (print, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &rv));
-	if (NS_FAILED(rv) || !print) return NS_ERROR_FAILURE;
-
-	rv = print->PrintPreviewNavigate(navType, pageNum);
-	return rv;
+	return print->PrintPreviewNavigate(navType, pageNum);
 }
 
 nsresult EphyBrowser::GetPrintSettings (nsIPrintSettings **options)
 {
-	nsresult result;
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &result));
-	if (NS_FAILED(result) || !print) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	NS_ENSURE_TRUE (print, NS_ERROR_FAILURE);
 
 	return print->GetGlobalPrintSettings(options);
 }
@@ -373,14 +374,14 @@ nsresult EphyBrowser::GetSHistory (nsISHistory **aSHistory)
 {
 	nsresult result;
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIWebNavigation> ContentNav = do_QueryInterface (mWebBrowser);
-	if (!ContentNav) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (ContentNav, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsISHistory> SessionHistory;
 	result = ContentNav->GetSessionHistory (getter_AddRefs (SessionHistory));
-	if (!SessionHistory) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (SessionHistory, NS_ERROR_FAILURE);
 
 	*aSHistory = SessionHistory.get();
 	NS_IF_ADDREF (*aSHistory);
@@ -401,29 +402,26 @@ nsresult EphyBrowser::GoToHistoryIndex (PRInt16 index)
 {
 	nsresult result;
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIWebNavigation> ContentNav = do_QueryInterface (mWebBrowser);
-	if (!ContentNav) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (ContentNav, NS_ERROR_FAILURE);
 
-	return  ContentNav->GotoIndex (index);
+	return ContentNav->GotoIndex (index);
 }
 
 nsresult EphyBrowser::SetZoom (float aZoom, PRBool reflow)
 {
-	nsresult result;
-
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	if (reflow)
 	{
 		nsCOMPtr<nsIContentViewer> contentViewer;	
-		result = GetContentViewer (getter_AddRefs(contentViewer));
-		if (!NS_SUCCEEDED (result) || !contentViewer) return NS_ERROR_FAILURE;
+		GetContentViewer (getter_AddRefs(contentViewer));
+		NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
 
-		nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer,
-								  &result);
-		if (NS_FAILED(result) || !mdv) return NS_ERROR_FAILURE;
+		nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
+		NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
 
 		return mdv->SetTextZoom (aZoom);
 	}
@@ -431,7 +429,7 @@ nsresult EphyBrowser::SetZoom (float aZoom, PRBool reflow)
 	{
 		nsCOMPtr<nsIDocShell> DocShell;
 		DocShell = do_GetInterface (mWebBrowser);
-		if (!DocShell) return NS_ERROR_FAILURE;
+		NS_ENSURE_TRUE (DocShell, NS_ERROR_FAILURE);
 
 		SetZoomOnDocshell (aZoom, DocShell);
 
@@ -459,46 +457,41 @@ nsresult EphyBrowser::SetZoom (float aZoom, PRBool reflow)
 
 nsresult EphyBrowser::SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell)
 {
-	nsresult result;
-
 	nsCOMPtr<nsIPresContext> PresContext;
-	result = DocShell->GetPresContext (getter_AddRefs(PresContext));
-	if (NS_FAILED(result) || !PresContext) return NS_ERROR_FAILURE;
+	DocShell->GetPresContext (getter_AddRefs(PresContext));
+	NS_ENSURE_TRUE (PresContext, NS_ERROR_FAILURE);
 					
 	nsCOMPtr<nsIDeviceContext> DeviceContext;
-	result = PresContext->GetDeviceContext (getter_AddRefs(DeviceContext));
-	if (NS_FAILED(result) || !DeviceContext) return NS_ERROR_FAILURE;
+	PresContext->GetDeviceContext (getter_AddRefs(DeviceContext));
+	NS_ENSURE_TRUE (DeviceContext, NS_ERROR_FAILURE);
 
 	return DeviceContext->SetTextZoom (aZoom);
 }
 
 nsresult EphyBrowser::GetContentViewer (nsIContentViewer **aViewer)
 {
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIDocShell> ourDocShell(do_GetInterface(mWebBrowser));
-	NS_ENSURE_TRUE(ourDocShell, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (ourDocShell, NS_ERROR_FAILURE);
+
 	return ourDocShell->GetContentViewer(aViewer);
 }
 
 nsresult EphyBrowser::GetZoom (float *aZoom)
 {
-	nsresult result;
-
 	nsCOMPtr<nsIContentViewer> contentViewer;	
-	result = GetContentViewer (getter_AddRefs(contentViewer));
-	if (!NS_SUCCEEDED (result) || !contentViewer) return NS_ERROR_FAILURE;
+	GetContentViewer (getter_AddRefs(contentViewer));
+	NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer,
-								  &result);
-	if (NS_FAILED(result) || !mdv) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
+	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
 
 	return mdv->GetTextZoom (aZoom);
 }
 
 nsresult EphyBrowser::GetDocument (nsIDOMDocument **aDOMDocument)
 {
-	nsCOMPtr<nsIDOMDocument> domDocument;
 	return mDOMWindow->GetDocument (aDOMDocument);
 }
 
@@ -506,7 +499,7 @@ nsresult EphyBrowser::GetTargetDocument (nsIDOMDocument **aDOMDocument)
 {
 	nsresult result;
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	/* Use the current target document */
 	if (mTargetDocument)
@@ -521,7 +514,7 @@ nsresult EphyBrowser::GetTargetDocument (nsIDOMDocument **aDOMDocument)
 	/* Use the focused document */
 	nsCOMPtr<nsIWebBrowserFocus> webBrowserFocus;
 	webBrowserFocus = do_QueryInterface (mWebBrowser);
-	if (!webBrowserFocus) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (webBrowserFocus, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIDOMWindow> DOMWindow;
 	result = webBrowserFocus->GetFocusedWindow (getter_AddRefs(DOMWindow));
@@ -536,11 +529,9 @@ nsresult EphyBrowser::GetTargetDocument (nsIDOMDocument **aDOMDocument)
 
 nsresult EphyBrowser::GetSHInfo (PRInt32 *count, PRInt32 *index)
 {
-	nsresult result;
-
 	nsCOMPtr<nsISHistory> SessionHistory;
-	result = GetSHistory (getter_AddRefs(SessionHistory));
-	if (NS_FAILED(result) || ! SessionHistory) return NS_ERROR_FAILURE;
+	GetSHistory (getter_AddRefs(SessionHistory));
+	NS_ENSURE_TRUE (SessionHistory, NS_ERROR_FAILURE);
 
 	SessionHistory->GetCount (count);
 	SessionHistory->GetIndex (index);	
@@ -550,42 +541,40 @@ nsresult EphyBrowser::GetSHInfo (PRInt32 *count, PRInt32 *index)
 
 nsresult EphyBrowser::GetSHTitleAtIndex (PRInt32 index, PRUnichar **title)
 {
-	nsresult result;
-
 	nsCOMPtr<nsISHistory> SessionHistory;
-	result = GetSHistory (getter_AddRefs(SessionHistory));
-	if (NS_FAILED(result) || ! SessionHistory) return NS_ERROR_FAILURE;
+	GetSHistory (getter_AddRefs(SessionHistory));
+	NS_ENSURE_TRUE (SessionHistory, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIHistoryEntry> he;
-	result = SessionHistory->GetEntryAtIndex (index, PR_FALSE,
-						  getter_AddRefs (he));
-	if (!NS_SUCCEEDED(result) || (!he)) return NS_ERROR_FAILURE;
+	SessionHistory->GetEntryAtIndex (index, PR_FALSE,
+					 getter_AddRefs (he));
+	NS_ENSURE_TRUE (he, NS_ERROR_FAILURE);
 
+	nsresult result;
 	result = he->GetTitle (title);
-	if (!NS_SUCCEEDED(result) || (!title)) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (NS_SUCCEEDED (result) && title, NS_ERROR_FAILURE);
 
 	return NS_OK;
 }
 
 nsresult EphyBrowser::GetSHUrlAtIndex (PRInt32 index, nsCString &url)
 {
-	nsresult result;
-
 	nsCOMPtr<nsISHistory> SessionHistory;
-	result = GetSHistory (getter_AddRefs(SessionHistory));
-	if (NS_FAILED(result) || ! SessionHistory) return NS_ERROR_FAILURE;
+	GetSHistory (getter_AddRefs(SessionHistory));
+	NS_ENSURE_TRUE (SessionHistory, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIHistoryEntry> he;
-	result = SessionHistory->GetEntryAtIndex (index, PR_FALSE,
-						  getter_AddRefs (he));
-	if (NS_FAILED(result) || (!he)) return NS_ERROR_FAILURE;
+	SessionHistory->GetEntryAtIndex (index, PR_FALSE,
+					 getter_AddRefs (he));
+	NS_ENSURE_TRUE (he, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIURI> uri;
-	result = he->GetURI (getter_AddRefs(uri));
-	if (NS_FAILED(result) || (!uri)) return NS_ERROR_FAILURE;
+	he->GetURI (getter_AddRefs(uri));
+	NS_ENSURE_TRUE (uri, NS_ERROR_FAILURE);
 
+	nsresult result;
 	result = uri->GetSpec(url);
-	if (NS_FAILED(result) || url.IsEmpty()) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (NS_SUCCEEDED (result) && !url.IsEmpty(), NS_ERROR_FAILURE);
 
 	return NS_OK;
 }
@@ -594,10 +583,11 @@ nsresult EphyBrowser::FindSetProperties (const PRUnichar *search_string,
 			                 PRBool case_sensitive,
 					 PRBool wrap_around)
 {
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIWebBrowserFind> finder (do_GetInterface(mWebBrowser));
-	
+	NS_ENSURE_TRUE (finder, NS_ERROR_FAILURE);
+
 	finder->SetSearchString (search_string);
 	finder->SetMatchCase (case_sensitive);
 	finder->SetWrapFind (wrap_around);
@@ -608,10 +598,11 @@ nsresult EphyBrowser::FindSetProperties (const PRUnichar *search_string,
 nsresult EphyBrowser::Find (PRBool backwards,
 			    PRBool *didFind)
 {
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIWebBrowserFind> finder (do_GetInterface(mWebBrowser));
-	
+	NS_ENSURE_TRUE (finder, NS_ERROR_FAILURE);
+
 	finder->SetFindBackwards (backwards);
 
 	return finder->FindNext(didFind);
@@ -621,13 +612,12 @@ nsresult EphyBrowser::GetPageDescriptor(nsISupports **aPageDescriptor)
 {
 	nsresult rv;
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIDocShell> ds;
-	ds = do_GetInterface (mWebBrowser);
+	nsCOMPtr<nsIDocShell> ds = do_GetInterface (mWebBrowser);
 
 	nsCOMPtr<nsIWebPageDescriptor> wpd = do_QueryInterface (ds);
-	if (!wpd) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (wpd, NS_ERROR_FAILURE);
 
 	*aPageDescriptor = wpd.get();
 	NS_IF_ADDREF (*aPageDescriptor);
@@ -637,68 +627,67 @@ nsresult EphyBrowser::GetPageDescriptor(nsISupports **aPageDescriptor)
 
 nsresult EphyBrowser::GetDocumentUrl (nsCString &url)
 {
-	nsresult result;
-
 	nsCOMPtr<nsIDOMDocument> DOMDocument;
-
-	result = mDOMWindow->GetDocument (getter_AddRefs(DOMDocument));
-	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
+	mDOMWindow->GetDocument (getter_AddRefs(DOMDocument));
+	NS_ENSURE_TRUE (DOMDocument, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
-	if(!doc) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 11
+#if MOZILLA_SNAPSHOT > 13
+	nsIURI *uri;
+	uri = doc->GetDocumentURI ();
+#elif MOZILLA_SNAPSHOT > 11
 	nsIURI *uri;
 	uri = doc->GetDocumentURL ();
 #else
 	nsCOMPtr<nsIURI> uri;
 	doc->GetDocumentURL(getter_AddRefs(uri));
 #endif
-	if (!uri) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (uri, NS_ERROR_FAILURE);
 
 	return uri->GetSpec (url);
 }
 
 nsresult EphyBrowser::GetTargetDocumentUrl (nsCString &url)
 {
-        nsresult result;
-
         nsCOMPtr<nsIDOMDocument> DOMDocument;
-
-        result = GetTargetDocument (getter_AddRefs(DOMDocument));
-        if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
+	GetTargetDocument (getter_AddRefs(DOMDocument));
+	NS_ENSURE_TRUE (DOMDocument, NS_ERROR_FAILURE);
 
         nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
-        if(!doc) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 11
+#if MOZILLA_SNAPSHOT > 13
+	nsIURI *uri;
+	uri = doc->GetDocumentURI ();
+#elif MOZILLA_SNAPSHOT > 11
 	nsIURI *uri;
 	uri = doc->GetDocumentURL ();
 #else
         nsCOMPtr<nsIURI> uri;
         doc->GetDocumentURL(getter_AddRefs(uri));
 #endif
-	if (!uri) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (uri, NS_ERROR_FAILURE);
 
 	return uri->GetSpec (url);
 }
 
 nsresult EphyBrowser::ForceEncoding (const char *encoding) 
 {
-	nsresult result;
-
 	nsCOMPtr<nsIContentViewer> contentViewer;	
-	result = GetContentViewer (getter_AddRefs(contentViewer));
-	if (!NS_SUCCEEDED (result) || !contentViewer) return NS_ERROR_FAILURE;
+	GetContentViewer (getter_AddRefs(contentViewer));
+	NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
-	if (!mdv) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
 
-	#if MOZILLA_SNAPSHOT > 9 
-		result = mdv->SetForceCharacterSet (nsDependentCString(encoding));
-	#else
-		result = mdv->SetForceCharacterSet (NS_ConvertUTF8toUCS2(encoding).get());
-	#endif
+	nsresult result;
+#if MOZILLA_SNAPSHOT > 9 
+	result = mdv->SetForceCharacterSet (nsDependentCString(encoding));
+#else
+	result = mdv->SetForceCharacterSet (NS_ConvertUTF8toUCS2(encoding).get());
+#endif
 
 	return result;
 }
@@ -723,11 +712,11 @@ nsresult EphyBrowser::GetEncodingInfo (EphyEncodingInfo **infoptr)
 	EphyEncodingInfo *info;
 
 	nsCOMPtr<nsIDOMDocument> domDoc;
-	result = GetTargetDocument (getter_AddRefs(domDoc));
-	if (NS_FAILED (result) || !domDoc) return NS_ERROR_FAILURE;
+	GetTargetDocument (getter_AddRefs(domDoc));
+	NS_ENSURE_TRUE (domDoc, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc, &result);
-	if (NS_FAILED (result) || !doc) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
 	info = g_new0 (EphyEncodingInfo, 1);
 	*infoptr = info;
@@ -743,15 +732,14 @@ nsresult EphyBrowser::GetEncodingInfo (EphyEncodingInfo **infoptr)
 
 	nsCOMPtr<nsIDocShell> ds;
 	ds = do_GetInterface (mWebBrowser);
-	if (!ds) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (ds, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIDocumentCharsetInfo> ci;
 	result = ds->GetDocumentCharsetInfo (getter_AddRefs (ci));
-	if (NS_FAILED(result) || !ci) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (ci, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIAtom> atom;
-	result = ci->GetForcedCharset (getter_AddRefs (atom));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+	ci->GetForcedCharset (getter_AddRefs (atom));
 	if (atom)
 	{
 		nsCAutoString atomstr;
@@ -759,8 +747,7 @@ nsresult EphyBrowser::GetEncodingInfo (EphyEncodingInfo **infoptr)
 		info->forced_encoding = g_strdup (atomstr.get());
 	}
 
-	result = ci->GetParentCharset (getter_AddRefs (atom));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+	ci->GetParentCharset (getter_AddRefs (atom));
 	if (atom)
 	{
 		nsCAutoString atomstr;
@@ -769,20 +756,19 @@ nsresult EphyBrowser::GetEncodingInfo (EphyEncodingInfo **infoptr)
 	}
 
 	result = ci->GetParentCharsetSource (&source);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
 	info->parent_encoding_source = (EphyEncodingSource) source;
 
 	nsCOMPtr<nsIContentViewer> contentViewer;	
-	result = ds->GetContentViewer (getter_AddRefs(contentViewer));
-	if (!NS_SUCCEEDED (result) || !contentViewer) return NS_ERROR_FAILURE;
+	ds->GetContentViewer (getter_AddRefs(contentViewer));
+	NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer,
-								  &result);
-	if (NS_FAILED(result) || !mdv) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
+	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
 
 #if MOZILLA_SNAPSHOT > 11
 	const nsACString& charsetEnc = doc->GetDocumentCharacterSet ();
-	if (charsetEnc.IsEmpty()) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (!charsetEnc.IsEmpty(), NS_ERROR_FAILURE);
 
 	info->encoding = g_strdup (PromiseFlatCString(charsetEnc).get());
 #elif MOZILLA_SNAPSHOT >= 10
@@ -803,19 +789,19 @@ nsresult EphyBrowser::GetEncodingInfo (EphyEncodingInfo **infoptr)
 	nsCAutoString enc;
 	
 	result = mdv->GetDefaultCharacterSet (enc);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
 	info->default_encoding = g_strdup (enc.get());
 
 	result = mdv->GetForceCharacterSet (enc);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
 	info->forced_encoding = g_strdup (enc.get());
 
 	result = mdv->GetHintCharacterSet (enc);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
 	info->hint_encoding = g_strdup (enc.get());
 
 	result = mdv->GetPrevDocCharacterSet (enc);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
 	info->prev_doc_encoding = g_strdup (enc.get());
 #else
 	PRUnichar *str;
@@ -838,7 +824,7 @@ nsresult EphyBrowser::GetEncodingInfo (EphyEncodingInfo **infoptr)
 #endif
 
 	mdv->GetHintCharacterSetSource (&source);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
 	info->hint_encoding_source = (EphyEncodingSource) source;
 
 	return NS_OK;
@@ -848,10 +834,10 @@ nsresult EphyBrowser::DoCommand (const char *command)
 {
 	nsCOMPtr<nsICommandManager> cmdManager;
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	cmdManager = do_GetInterface (mWebBrowser);
-	if (!cmdManager) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (cmdManager, NS_ERROR_FAILURE);
 
 	return cmdManager->DoCommand (command, nsnull, nsnull);
 }
@@ -860,10 +846,134 @@ nsresult EphyBrowser::GetCommandState (const char *command, PRBool *enabled)
 {
 	nsCOMPtr<nsICommandManager> cmdManager;
 
-	g_return_val_if_fail (mWebBrowser, NS_ERROR_FAILURE);
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	cmdManager = do_GetInterface (mWebBrowser);
-	if (!cmdManager) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (cmdManager, NS_ERROR_FAILURE);
 
 	return cmdManager->IsCommandEnabled (command, nsnull, enabled);
 }
+
+#define NUM_MODIFIED_TEXTFIELDS_REQUIRED	2
+
+nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUint32 *aNumTextFields, PRBool *aHasTextArea)
+{
+	nsCOMPtr<nsIDOMHTMLDocument> htmlDoc = do_QueryInterface(aDomDoc);
+	NS_ENSURE_TRUE (htmlDoc, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIDOMHTMLCollection> forms;
+	htmlDoc->GetForms (getter_AddRefs (forms));
+	if (!forms) return NS_OK; /* it's ok not to have any forms */
+
+	PRUint32 formNum;
+	forms->GetLength (&formNum);
+
+	/* check all forms */
+	for (PRUint32 formIndex = 0; formIndex < formNum; formIndex++)
+	{
+		nsCOMPtr<nsIDOMNode> formNode;
+		forms->Item (formIndex, getter_AddRefs (formNode));
+		if (!formNode) continue;
+
+		nsCOMPtr<nsIDOMHTMLFormElement> formElement = do_QueryInterface (formNode);
+		if (!formElement) continue;
+
+		nsCOMPtr<nsIDOMHTMLCollection> formElements;
+		formElement->GetElements (getter_AddRefs (formElements));
+		if (!formElements) continue;
+
+		PRUint32 elementNum;
+		formElements->GetLength (&elementNum);
+
+		/* check all input elements in the form for user input */
+		for (PRUint32 elementIndex = 0; elementIndex < elementNum; elementIndex++)
+		{
+			nsCOMPtr<nsIDOMNode> domNode;
+			formElements->Item (elementIndex, getter_AddRefs (domNode));
+			if (!domNode) continue;
+
+			nsCOMPtr<nsIDOMHTMLTextAreaElement> areaElement = do_QueryInterface (domNode);
+			if (areaElement)
+			{
+				nsAutoString default_text, user_text;
+				areaElement->GetDefaultValue (default_text);
+				areaElement->GetValue (user_text);
+				if (Compare (user_text, default_text) != 0)
+				{
+					*aHasTextArea = PR_TRUE;
+					return NS_OK;
+				}
+
+				continue;
+			}
+
+			nsCOMPtr<nsIDOMHTMLInputElement> inputElement = do_QueryInterface(domNode);
+			if (!inputElement) continue;
+	
+			nsAutoString type;
+			inputElement->GetType(type);
+
+			if (type.EqualsIgnoreCase("text"))
+			{
+				nsAutoString default_text, user_text;
+				inputElement->GetDefaultValue (default_text);
+				inputElement->GetValue (user_text);
+				if (Compare (user_text, default_text) != 0)
+				{
+					(*aNumTextFields)++;
+					if (*aNumTextFields >= NUM_MODIFIED_TEXTFIELDS_REQUIRED)
+					{
+						return NS_OK;
+					}
+				}
+			}
+		}
+	}
+
+	return NS_OK;
+}
+
+nsresult EphyBrowser::GetHasModifiedForms (PRBool *modified)
+{
+	*modified = PR_FALSE;
+
+	nsCOMPtr<nsIDocShell> rootDocShell = do_GetInterface (mWebBrowser);
+	NS_ENSURE_TRUE (rootDocShell, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsISimpleEnumerator> enumerator;
+	rootDocShell->GetDocShellEnumerator(nsIDocShellTreeItem::typeContent,
+					    nsIDocShell::ENUMERATE_FORWARDS,
+					    getter_AddRefs(enumerator));
+	NS_ENSURE_TRUE (enumerator, NS_ERROR_FAILURE);
+
+	PRBool hasMore;
+	PRBool hasTextArea = PR_FALSE;
+	PRUint32 numTextFields = 0;
+	while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMore)) && hasMore)
+	{
+		nsCOMPtr<nsISupports> element;
+		enumerator->GetNext (getter_AddRefs(element));
+		if (!element) continue;
+
+		nsCOMPtr<nsIDocShell> docShell = do_QueryInterface (element);
+		if (!docShell) continue;
+
+		nsCOMPtr<nsIContentViewer> contentViewer;
+		docShell->GetContentViewer (getter_AddRefs(contentViewer));
+		if (!contentViewer) continue;
+
+		nsCOMPtr<nsIDOMDocument> domDoc;
+		contentViewer->GetDOMDocument (getter_AddRefs (domDoc));
+
+		nsresult result;
+		result = GetDocumentHasModifiedForms (domDoc, &numTextFields, &hasTextArea);
+		if (NS_SUCCEEDED (result) &&
+		    (numTextFields >= NUM_MODIFIED_TEXTFIELDS_REQUIRED || hasTextArea))
+		{
+			*modified = PR_TRUE;
+			break;
+		}
+	}
+
+	return NS_OK;
+}
