@@ -36,7 +36,7 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: EphyHeaderSniffer.cpp,v 1.15 2003/11/27 23:46:32 xan Exp $
+ * $Id: EphyHeaderSniffer.cpp,v 1.16 2003/12/29 21:05:06 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,6 +49,8 @@
 #include "EphyHeaderSniffer.h"
 #include "netCore.h"
 
+#include "ephy-embed-single.h"
+#include "ephy-embed-shell.h"
 #include "ephy-file-chooser.h"
 #include "ephy-prefs.h"
 #include "ephy-gui.h"
@@ -101,7 +103,7 @@ EphyHeaderSniffer::HandleContent ()
 	if (mPostData) return NS_ERROR_FAILURE;
 
 	mURL->GetSpec (uriSpec);	
-	single = ephy_embed_shell_get_embed_single (embed_shell);
+	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
 	g_signal_emit_by_name (single, "handle_content", mContentType.get(),
 			       uriSpec.get(), &handled);
 
