@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.cpp,v 1.12 2003/10/15 19:16:10 chpe Exp $
+ *  $Id: EventContext.cpp,v 1.20 2004/01/12 19:31:40 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,6 +32,7 @@
 #include "nsIDOMElement.h"
 #include "nsIDOMXULDocument.h"
 #include "nsIURI.h"
+#include "nsNetUtil.h"
 #include "nsIDOMNSDocument.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
@@ -56,9 +57,9 @@ EventContext::~EventContext ()
 {
 }
 
-nsresult EventContext::Init (EphyWrapper *wrapper)
+nsresult EventContext::Init (EphyBrowser *browser)
 {
-	mWrapper = wrapper;
+	mBrowser = browser;
 	mDOMDocument = nsnull;
 
 	return NS_OK;
@@ -145,13 +146,23 @@ nsresult EventContext::GatherTextUnder (nsIDOMNode* aNode, nsString& aResult)
 nsresult EventContext::ResolveBaseURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
 {
 	nsresult rv;
+#if MOZILLA_SNAPSHOT > 13
+	nsIURI *base;
+	base = doc->GetBaseURI ();
+	if (!base) return NS_ERROR_FAILURE;
+#elif MOZILLA_SNAPSHOT > 11
+	nsIURI *base;
+	base = doc->GetBaseURL ();
+	if (!base) return NS_ERROR_FAILURE;
+#elif MOZILLA_SNAPSHOT > 9 
 	nsCOMPtr<nsIURI> base;
-#if MOZILLA_SNAPSHOT > 9 
 	rv = doc->GetBaseURL (getter_AddRefs(base));
-#else
-	rv = doc->GetBaseURL (*getter_AddRefs(base));
-#endif
 	if (NS_FAILED(rv)) return rv;
+#else
+	nsCOMPtr<nsIURI> base;
+	rv = doc->GetBaseURL (*getter_AddRefs(base));
+	if (NS_FAILED(rv)) return rv;
+#endif
 
 	return base->Resolve (NS_ConvertUCS2toUTF8(relurl), url);
 }
@@ -159,9 +170,19 @@ nsresult EventContext::ResolveBaseURL (nsIDocument *doc, const nsAString &relurl
 nsresult EventContext::ResolveDocumentURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
 {
 	nsresult rv;
+#if MOZILLA_SNAPSHOT > 13
+	nsIURI *uri;
+	uri = doc->GetDocumentURI ();
+	if (!uri) return NS_ERROR_FAILURE;
+#elif MOZILLA_SNAPSHOT > 11
+	nsIURI *uri;
+	uri = doc->GetDocumentURL ();
+	if (!uri) return NS_ERROR_FAILURE;
+#else
 	nsCOMPtr<nsIURI> uri;
 	rv = doc->GetDocumentURL(getter_AddRefs(uri));
 	if (NS_FAILED(rv)) return rv;
+#endif
 
 	return uri->Resolve (NS_ConvertUCS2toUTF8(relurl), url);
 }
@@ -369,6 +390,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				dom_elem->GetAttributeNS (nspace, localname_href, value);
 				
 				SetStringProperty ("link", value);
+				CheckLinkScheme (value);
 			}
 		}
 
@@ -412,6 +434,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					info->context |= EMBED_CONTEXT_LINK;
 
 					SetStringProperty ("link", tmp);
+					CheckLinkScheme (tmp);
 					rv = anchor->GetHreflang (tmp);
 					if (NS_SUCCEEDED(rv))
 						SetStringProperty ("link_lang", tmp);
@@ -485,6 +508,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 						return NS_ERROR_FAILURE;
 					
 					SetStringProperty ("link", href);
+					CheckLinkScheme (href);
 				}
 			}
 			else if (tag.Equals(NS_LITERAL_STRING("textarea"),
@@ -686,7 +710,7 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, EphyEmb
 	if (mod_key) info->modifier |= GDK_SHIFT_MASK;
 
 	aMouseEvent->GetMetaKey(&mod_key);
-	if (mod_key) info->modifier |= GDK_Meta_L;
+	if (mod_key) info->modifier |= GDK_MOD2_MASK;
 	
 	aMouseEvent->GetCtrlKey(&mod_key);
 	if (mod_key) info->modifier |= GDK_CONTROL_MASK;
@@ -746,7 +770,7 @@ nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, EphyEmbedEven
 	if (mod_key) info->modifier |= GDK_SHIFT_MASK;
 
 	aKeyEvent->GetMetaKey(&mod_key);
-	if (mod_key) info->modifier |= GDK_Meta_L;
+	if (mod_key) info->modifier |= GDK_MOD2_MASK;
 	
 	aKeyEvent->GetCtrlKey(&mod_key);
 	if (mod_key) info->modifier |= GDK_CONTROL_MASK;
@@ -757,9 +781,9 @@ nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, EphyEmbedEven
 nsresult EventContext::IsPageFramed (nsIDOMNode *node, PRBool *Framed)
 {
 	nsresult result;
-	
+
 	nsCOMPtr<nsIDOMDocument> mainDocument;
-	result = mWrapper->GetMainDOMDocument (getter_AddRefs(mainDocument));
+	result = mBrowser->GetDocument (getter_AddRefs(mainDocument));
 	if (NS_FAILED(result) || !mainDocument) return NS_ERROR_FAILURE;
 	
 	nsCOMPtr<nsIDOMDocument> nodeDocument;
@@ -782,6 +806,29 @@ nsresult EventContext::GetTargetDocument (nsIDOMDocument **domDoc)
 	return NS_OK;
 }
 
+nsresult EventContext::CheckLinkScheme (const nsAString &link)
+{
+	nsCOMPtr<nsIURI> uri;
+	NS_NewURI (getter_AddRefs (uri), link);
+	if (!uri) return NS_ERROR_FAILURE;
+
+	nsresult rv;
+	nsCAutoString scheme;
+	rv = uri->GetScheme (scheme);
+	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+
+	if (scheme.EqualsIgnoreCase ("http") ||
+	    scheme.EqualsIgnoreCase ("https") ||
+	    scheme.EqualsIgnoreCase ("ftp") ||
+	    scheme.EqualsIgnoreCase ("file") ||
+	    scheme.EqualsIgnoreCase ("gopher"))
+	{
+		SetIntProperty ("link-has-web-scheme", TRUE);
+	}
+
+	return NS_OK;
+}
+
 nsresult EventContext::SetIntProperty (const char *name, int value)
 {
 
