@@ -31,7 +31,7 @@
 #include "nsIDOMElementCSSInlineStyle.h"
 #include "nsIDOMCSSStyleDeclaration.h"
 #include "nsIDOMDocument.h"
-#include "EphyWrapper.h"
+#include "EphyBrowser.h"
 
 #include "ephy-embed.h"
 #include "ephy-embed-event.h"
@@ -42,22 +42,24 @@ public:
 	EventContext();
 	~EventContext();
 
-	nsresult Init (EphyWrapper *wrapper);
+	nsresult Init (EphyBrowser *wrapper);
 
 	nsresult GetMouseEventInfo (nsIDOMMouseEvent *event, EphyEmbedEvent *info);
 	nsresult GetKeyEventInfo (nsIDOMKeyEvent *event, EphyEmbedEvent *info);
 	nsresult GetTargetDocument (nsIDOMDocument **domDoc);
 
 private:
-	EphyWrapper *mWrapper;
+	EphyBrowser *mBrowser;
 	nsCOMPtr<nsIDOMDocument> mDOMDocument;
 
+	nsresult GatherTextUnder (nsIDOMNode* aNode, nsString& aResult);
 	nsresult ResolveBaseURL (nsIDocument *doc, const nsAString &relurl, nsACString &url);
 	nsresult ResolveDocumentURL (nsIDocument *doc, const nsAString &relurl, nsACString &url);
 	nsresult GetEventContext (nsIDOMEventTarget *EventTarget,
 				  EphyEmbedEvent *info);
 	nsresult GetCSSBackground (nsIDOMNode *node, nsAutoString& url);
 	nsresult IsPageFramed (nsIDOMNode *node, PRBool *Framed);
+	nsresult CheckLinkScheme (const nsAString &link);
 	nsresult SetIntProperty (const char *name, int value);
 	nsresult SetStringProperty (const char *name, const char *value);
 	nsresult SetStringProperty (const char *name, const nsAString &value);
