@@ -38,9 +38,13 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp,v 1.9 2003/11/27 23:46:32 xan Exp $
+ * $Id: MozDownload.cpp,v 1.14 2004/01/10 11:19:52 chpe Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "MozDownload.h"
 #include "mozilla-download.h"
 #include "eel-gconf-extensions.h"
@@ -52,13 +56,11 @@
 #include "nsDirectoryServiceUtils.h"
 #include "nsIRequest.h"
 #include "netCore.h"
-#include "nsIObserver.h"
 
 const char* const persistContractID = "@mozilla.org/embedding/browser/nsWebBrowserPersist;1";
 
 MozDownload::MozDownload() :
 	mGotFirstStateChange(false), mIsNetworkTransfer(false),
-	mUserCanceled(false),
 	mStatus(NS_OK),
 	mEmbedPersist(nsnull),
 	mDownloadState(EPHY_DOWNLOAD_DOWNLOADING)
@@ -251,7 +253,9 @@ MozDownload::GetObserver(nsIObserver **aObserver)
 NS_IMETHODIMP
 MozDownload::SetObserver(nsIObserver *aObserver)
 {
-	return NS_ERROR_NOT_IMPLEMENTED;
+	mObserver = aObserver;
+
+	return NS_OK;
 }
 
 NS_IMETHODIMP 
@@ -308,6 +312,9 @@ MozDownload::OnProgressChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest
 			      PRInt32 aCurSelfProgress, PRInt32 aMaxSelfProgress,
 			      PRInt32 aCurTotalProgress, PRInt32 aMaxTotalProgress)
 {
+	if (!mRequest)
+		mRequest = aRequest;
+
 	PRInt64 now = PR_Now ();
 
 	if ((now - mLastUpdate < mInterval) &&
@@ -316,15 +323,6 @@ MozDownload::OnProgressChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest
 		return NS_OK;
 
 	mLastUpdate = now;
-    
-	if (mUserCanceled)
-	{
-	        if (aRequest)
-		{
-	            aRequest->Cancel(NS_BINDING_ABORTED);
-		}
-	        mUserCanceled = false;
-	}
 
 	if (aMaxTotalProgress == -1)
 	{
@@ -368,21 +366,35 @@ MozDownload::OnSecurityChange (nsIWebProgress *aWebProgress, nsIRequest *aReques
 void
 MozDownload::Cancel()
 {
-    mUserCanceled = true;
-    /* nsWebBrowserPersist does the right thing: After canceling, next time through
-       OnStateChange(), aStatus != NS_OK. This isn't the case with nsExternalHelperAppService.*/
-    if (!mWebPersist)
-        mStatus = NS_ERROR_ABORT;
+	if (mWebPersist)
+	{
+		mWebPersist->CancelSave ();
+	}
+
+	if (mObserver)
+	{
+		mObserver->Observe (nsnull, "oncancel", nsnull);
+	}	
 }
 
 void
 MozDownload::Pause()
 {
+	if (mRequest)
+	{
+		mRequest->Suspend ();
+		mDownloadState = EPHY_DOWNLOAD_PAUSED;
+	}
 }
 
 void
 MozDownload::Resume()
 {
+	if (mRequest)
+	{
+		mRequest->Resume ();
+		mDownloadState = EPHY_DOWNLOAD_DOWNLOADING;
+	}
 }
 
 nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI,
@@ -401,7 +413,7 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 			 strcmp (contentType, "application/xhtml+xml") == 0));
 
 	nsCOMPtr<nsIWebBrowserPersist> webPersist = do_CreateInstance(persistContractID, &rv);
-	if (NS_FAILED(rv)) return rv;
+	NS_ENSURE_SUCCESS (rv, rv);
   
 	PRInt64 timeNow = PR_Now();
   
@@ -412,7 +424,7 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 	/* dlListener attaches to its progress dialog here, which gains ownership */
 	rv = downloader->InitForEmbed (inOriginalURI, inDestFile, fileDisplayName.get(),
 				       nsnull, timeNow, webPersist, embedPersist);
-	if (NS_FAILED(rv)) return rv;
+	NS_ENSURE_SUCCESS (rv, rv);
 
 	PRInt32 flags = nsIWebBrowserPersist::PERSIST_FLAGS_REPLACE_EXISTING_FILES;
 	if (ephy_flags & EMBED_PERSIST_COPY_PAGE)
@@ -507,10 +519,11 @@ GetFilePath (const char *filename)
 	}
 	else
 	{
-		path = g_build_filename
-			(gnome_vfs_expand_initial_tilde (download_dir),
-			 filename,
-			 NULL);
+		char *expanded;
+
+		expanded = gnome_vfs_expand_initial_tilde (download_dir);
+		path = g_build_filename (expanded, filename, NULL);
+		g_free (expanded);
 	}
 	g_free (download_dir);
 
