@@ -50,6 +50,8 @@
 #include "nsIURI.h"
 #include "nsILocalFile.h"
 #include "nsIWebBrowserPersist.h"
+#include "nsIObserver.h"
+#include "nsIRequest.h"
 
 #include "downloader-view.h"
 #include "ephy-download.h"
