@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.c,v 1.68 2003/12/16 17:29:43 marco Exp $
+ *  $Id: ephy-bookmarks.c,v 1.71 2004/01/10 16:35:56 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,6 +24,7 @@
 
 #include "ephy-bookmarks.h"
 #include "ephy-file-helpers.h"
+#include "ephy-embed-shell.h"
 #include "ephy-shell.h"
 #include "ephy-history.h"
 #include "ephy-debug.h"
@@ -32,6 +33,9 @@
 #include "ephy-toolbars-model.h"
 #include "ephy-bookmarks-export.h"
 #include "ephy-bookmarks-import.h"
+#include "ephy-prefs.h"
+
+#include "eel-gconf-extensions.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
@@ -60,6 +64,8 @@ struct EphyBookmarksPrivate
 	EphyNode *smartbookmarks;
 	EphyNode *lower_fav;
 	double lower_score;
+
+	guint disable_bookmark_editing_notifier_id;
 };
 
 typedef struct
@@ -310,12 +316,10 @@ compute_lower_fav (EphyNode *favorites, double *score)
 {
 	GPtrArray *children;
 	int i;
-	EphyEmbedShell *embed_shell;
 	EphyHistory *history;
 	EphyNode *result = NULL;
 
-	embed_shell = EPHY_EMBED_SHELL (ephy_shell);
-	history = ephy_embed_shell_get_global_history (embed_shell);
+	history = EPHY_HISTORY (ephy_embed_shell_get_global_history (embed_shell));
 
 	*score = DBL_MAX;
 	children = ephy_node_get_children (favorites);
@@ -386,17 +390,57 @@ history_site_visited_cb (EphyHistory *gh, const char *url, EphyBookmarks *eb)
 	add_to_favorites (eb, node, gh);
 }
 
+static void
+clear_favorites (EphyBookmarks *bookmarks)
+{
+	EphyNode *node;
+	GPtrArray *children;
+	int i;
+	gboolean was_immutable;
+
+	/* clear the favourites */
+
+	was_immutable = ephy_node_db_is_immutable (bookmarks->priv->db);
+	ephy_node_db_set_immutable (bookmarks->priv->db, FALSE);
+
+	node = bookmarks->priv->favorites;
+	children = ephy_node_get_children (node);
+	for (i = children->len - 1; i >= 0; i--)
+	{
+		EphyNode *kid;
+
+		kid = g_ptr_array_index (children, i);
+
+		ephy_node_remove_child (node, kid);
+	}
+
+	ephy_node_db_set_immutable (bookmarks->priv->db, was_immutable);
+
+	ephy_bookmarks_update_favorites (bookmarks);
+}
+
+static void
+history_cleared_cb (EphyHistory *history, EphyBookmarks *bookmarks)
+{
+	clear_favorites (bookmarks);
+}
+
 static void
 ephy_setup_history_notifiers (EphyBookmarks *eb)
 {
-	EphyEmbedShell *embed_shell;
 	EphyHistory *history;
 
-	embed_shell = EPHY_EMBED_SHELL (ephy_shell);
-	history = ephy_embed_shell_get_global_history (embed_shell);
+	history = EPHY_HISTORY (ephy_embed_shell_get_global_history (embed_shell));
+
+	if (ephy_history_is_enabled (history) == FALSE)
+	{
+		clear_favorites (eb);
+	}
 
 	g_signal_connect (history, "visited",
 			  G_CALLBACK (history_site_visited_cb), eb);
+	g_signal_connect (history, "cleared",
+			  G_CALLBACK (history_cleared_cb), eb);
 }
 
 static void
@@ -499,6 +543,23 @@ topics_removed_cb (EphyNode *node,
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
 
+static void
+update_bookmark_editing (EphyBookmarks *eb)
+{
+	g_object_set (G_OBJECT (eb->priv->db),
+		      "immutable", eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING),
+		      NULL);
+}
+
+static void
+disable_bookmark_editing_notifier (GConfClient *client,
+				   guint cnxn_id,
+				   GConfEntry *entry,
+				   EphyBookmarks *eb)
+{
+	update_bookmark_editing (eb);
+}
+
 static void
 ephy_bookmarks_init (EphyBookmarks *eb)
 {
@@ -602,6 +663,11 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 		}
 	}
 
+	eb->priv->disable_bookmark_editing_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING,
+		 (GConfClientNotifyFunc)disable_bookmark_editing_notifier, eb);
+	update_bookmark_editing (eb);
+
 	ephy_setup_history_notifiers (eb);
 	ephy_bookmarks_update_favorites (eb);
 }
@@ -611,6 +677,8 @@ ephy_bookmarks_finalize (GObject *object)
 {
 	EphyBookmarks *eb = EPHY_BOOKMARKS (object);
 
+	eel_gconf_notification_remove (eb->priv->disable_bookmark_editing_notifier_id);
+
 	if (eb->priv->save_timeout_id != 0)
 	{
 		g_source_remove (eb->priv->save_timeout_id);
@@ -688,6 +756,8 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 
 	bm = ephy_node_new (eb->priv->db);
 
+	if (bm == NULL) return NULL;
+
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, title);
 	ephy_node_set_property (bm, EPHY_NODE_BMK_PROP_TITLE,
@@ -941,6 +1011,8 @@ ephy_bookmarks_add_keyword (EphyBookmarks *eb,
 
 	key = ephy_node_new (eb->priv->db);
 
+	if (key == NULL) return NULL;
+
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, name);
 	ephy_node_set_property (key, EPHY_NODE_KEYWORD_PROP_NAME,
