@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-session.c,v 1.5 2003/11/20 18:05:15 marco Exp $
+ *  $Id: ephy-session.c,v 1.7 2004/01/10 15:35:19 clahey Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,6 +33,8 @@
 #include "ephy-bookmarks-editor.h"
 #include "ephy-string.h"
 #include "ephy-file-helpers.h"
+#include "eel-gconf-extensions.h"
+#include "ephy-prefs.h"
 #include "ephy-debug.h"
 
 #include <glib/gi18n.h>
@@ -646,11 +648,17 @@ ephy_session_load (EphySession *session,
 
 			if (id && xmlStrEqual (BOOKMARKS_EDITOR_ID, id))
 			{
-				widget = ephy_shell_get_bookmarks_editor (ephy_shell);
+				if (!eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING))
+				{
+					widget = ephy_shell_get_bookmarks_editor (ephy_shell);
+				}
 			}
 			else if (id && xmlStrEqual (HISTORY_WINDOW_ID, id))
 			{
-				widget = ephy_shell_get_history_window (ephy_shell);
+				if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_HISTORY))
+				{
+					widget = ephy_shell_get_history_window (ephy_shell);
+				}
 			}
 		}
 
