@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.80 2003/12/25 14:48:12 chpe Exp $
+ *  $Id: ephy-shell.c,v 1.81 2003/12/29 21:05:07 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -781,8 +781,8 @@ ephy_shell_get_history_window (EphyShell *shell)
 
 	if (shell->priv->history_window == NULL)
 	{
-		history = ephy_embed_shell_get_global_history
-			(EPHY_EMBED_SHELL (ephy_shell));
+		history = EPHY_HISTORY
+			(ephy_embed_shell_get_global_history (embed_shell));
 		g_assert (history != NULL);
 		shell->priv->history_window = ephy_history_window_new (history);
 
