@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.71 2003/12/08 15:21:31 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.80 2004/01/12 19:33:03 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,7 +24,6 @@
 #endif
 
 #include "ephy-tab.h"
-#include "ephy-shell.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-embed-factory.h"
@@ -36,6 +35,9 @@
 #include "ephy-zoom.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-embed-persist.h"
+#include "ephy-history.h"
+#include "ephy-embed-shell.h"
+#include "ephy-shell.h"
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -409,7 +411,8 @@ address_has_web_scheme (const char *address)
 		has_web_scheme = (strcmp (scheme, "http") == 0 ||
 				  strcmp (scheme, "https") == 0 ||
 				  strcmp (scheme, "ftp") == 0 ||
-				  strcmp (scheme, "file") == 0);
+				  strcmp (scheme, "file") == 0 ||
+				  strcmp (scheme, "gopher") == 0);
 
 		gnome_vfs_uri_unref (uri);
 	}
@@ -453,7 +456,7 @@ ephy_tab_set_link_message (EphyTab *tab, char *message)
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	g_free (tab->priv->link_message);
-	tab->priv->link_message = message;
+	tab->priv->link_message = ephy_string_blank_chr (message);
 
 	g_object_notify (G_OBJECT (tab), "message");
 }
@@ -474,6 +477,18 @@ ephy_tab_get_embed (EphyTab *tab)
 	return tab->priv->embed;
 }
 
+EphyTab *
+ephy_tab_for_embed (EphyEmbed *embed)
+{
+	EphyTab *tab;
+
+	g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
+
+	tab = EPHY_TAB (g_object_get_data (G_OBJECT (embed), "ephy-tab"));
+
+	return tab;
+}
+
 void
 ephy_tab_set_window (EphyTab *tab, EphyWindow *window)
 {
@@ -555,8 +570,8 @@ ephy_tab_set_icon_address (EphyTab *tab, const char *address)
 	if (tab->priv->icon_address)
 	{
 		eb = ephy_shell_get_bookmarks (ephy_shell);
-		history = ephy_embed_shell_get_global_history
-			(EPHY_EMBED_SHELL (ephy_shell));
+		history = EPHY_HISTORY
+			(ephy_embed_shell_get_global_history (embed_shell));
 		ephy_bookmarks_set_icon (eb, tab->priv->address,
 				         tab->priv->icon_address);
 		ephy_history_set_icon (history, tab->priv->address,
@@ -613,8 +628,8 @@ ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 		GValue value = { 0, };
 		float zoom = 1.0, current_zoom;
 
-		history = ephy_embed_shell_get_global_history
-				(EPHY_EMBED_SHELL (ephy_shell));
+		history = EPHY_HISTORY
+			(ephy_embed_shell_get_global_history (embed_shell));
 		host = ephy_history_get_host (history, address);
 
 		if (host != NULL && ephy_node_get_property
@@ -654,8 +669,8 @@ ephy_tab_zoom_changed_cb (EphyEmbed *embed, float zoom, EphyTab *tab)
 		EphyHistory *history;
 		EphyNode *host;
 		GValue value = { 0, };
-		history = ephy_embed_shell_get_global_history
-				(EPHY_EMBED_SHELL (ephy_shell));
+		history = EPHY_HISTORY
+			(ephy_embed_shell_get_global_history (embed_shell));
 		host = ephy_history_get_host (history, address);
 
 		if (host != NULL)
@@ -737,11 +752,11 @@ get_host_name_from_uri (const char *uri)
 	return result;
 }
 
-static char *
-build_net_state_message (const char *uri, EmbedState flags)
+static void
+update_net_state_message (EphyTab *tab, const char *uri, EmbedState flags)
 {
 	const char *msg = NULL;
-	char *host, *message = NULL;
+	char *host;
 
 	host = get_host_name_from_uri (uri);
 
@@ -769,20 +784,24 @@ build_net_state_message (const char *uri, EmbedState flags)
                 {
                         msg = _("Loading %s...");
                 }
-                else if (flags & EMBED_STATE_STOP)
-                {
-			msg = ("");
-                }
         }
 
-	if (msg)
+	if ((flags & EMBED_STATE_IS_NETWORK) &&
+	    (flags & EMBED_STATE_STOP))
+        {
+		g_free (tab->priv->status_message);
+		tab->priv->status_message = NULL;
+		g_object_notify (G_OBJECT (tab), "message");
+
+	}
+	else if (msg)
 	{
-		message = g_strdup_printf (msg, host); 
+		g_free (tab->priv->status_message);
+		tab->priv->status_message = g_strdup_printf (msg, host);
+		g_object_notify (G_OBJECT (tab), "message");
 	}
 
 	g_free (host);
-
-	return message;
 }
 
 static void
@@ -803,10 +822,8 @@ build_progress_from_requests (EphyTab *tab, EmbedState state)
 
 		load_percent = build_load_percent (tab->priv->cur_requests,
 						   tab->priv->total_requests);
-		if (load_percent > tab->priv->load_percent || load_percent == -1)
-		{
-			ephy_tab_set_load_percent (tab, load_percent);
-		}
+
+		ephy_tab_set_load_percent (tab, load_percent);
 	}
 }
 
@@ -829,16 +846,7 @@ static void
 ephy_tab_net_state_cb (EphyEmbed *embed, const char *uri,
 		       EmbedState state, EphyTab *tab)
 {
-	char *new_msg;
-
-	new_msg = build_net_state_message (uri, state);
-	if (new_msg)
-	{
-		g_free (tab->priv->status_message);
-		tab->priv->status_message = new_msg;
-	}
-
-	g_object_notify (G_OBJECT (tab), "message");
+	update_net_state_message (tab, uri, state);
 
 	if (state & EMBED_STATE_IS_NETWORK)
 	{
@@ -1011,6 +1019,7 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 	gboolean with_control, with_shift, is_left_click, is_middle_click;
 	gboolean is_link, is_image, is_middle_clickable;
 	gboolean middle_click_opens;
+	gboolean is_input;
 
 	g_return_val_if_fail (EPHY_IS_EMBED_EVENT(event), FALSE);
 
@@ -1021,19 +1030,24 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 	context = ephy_embed_event_get_context (event);
 	modifier = ephy_embed_event_get_modifier (event);
 
+	LOG ("ephy_tab_dom_mouse_click_cb: type %d, context %x, modifier %x",
+	     type, context, modifier)
+
 	with_control = (modifier & GDK_CONTROL_MASK) != 0;
 	with_shift = (modifier & GDK_SHIFT_MASK) != 0;
 	is_left_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON1);
 	is_middle_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2);
 
-	middle_click_opens = eel_gconf_get_boolean
-				(CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL);
+	middle_click_opens =
+		eel_gconf_get_boolean (CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL) &&
+		!eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL);
 
 	is_link = (context & EMBED_CONTEXT_LINK) != 0;
 	is_image = (context & EMBED_CONTEXT_IMAGE) != 0;
 	is_middle_clickable = !((context & EMBED_CONTEXT_LINK)
 				|| (context & EMBED_CONTEXT_INPUT)
 				|| (context & EMBED_CONTEXT_EMAIL_LINK));
+	is_input = (context & EMBED_CONTEXT_INPUT) != 0;
 
 	/* ctrl+click or middle click opens the link in new tab */
 	if (is_link && ((is_left_click && with_control) || is_middle_click))
@@ -1050,8 +1064,10 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 	{
 		save_property_url (embed, event, "link", CONF_STATE_DOWNLOAD_DIR);
 	}
-	/* shift+click saves the non-link image */
-	else if (is_image && is_left_click && with_shift)
+	/* shift+click saves the non-link image
+	 * Note: pressing enter to submit a form synthesizes a mouse click event
+	 */
+	else if (is_image && is_left_click && with_shift && !is_input)
 	{
 		save_property_url (embed, event, "image", CONF_STATE_SAVE_IMAGE_DIR);
 	}
@@ -1104,7 +1120,6 @@ ephy_tab_init (EphyTab *tab)
 	tab->priv->load_status = FALSE;
 	tab->priv->link_message = NULL;
 	tab->priv->security_level = STATE_IS_UNKNOWN;
-	tab->priv->status_message = NULL;
 	tab->priv->zoom = 1.0;
 	tab->priv->setting_zoom = FALSE;
 	tab->priv->address_expire = TAB_ADDRESS_EXPIRE_NOW;
@@ -1129,7 +1144,7 @@ ephy_tab_init (EphyTab *tab)
 			  G_CALLBACK (ephy_tab_action_activate_cb), tab);
 
 	/* set a pointer in the embed's widget back to the tab */
-	g_object_set_data (embed_widget, "EphyTab", tab);
+	g_object_set_data (embed_widget, "ephy-tab", tab);
 
 	g_signal_connect (embed_widget, "parent_set",
 			  G_CALLBACK (ephy_tab_parent_set_cb),
@@ -1244,7 +1259,7 @@ ephy_tab_get_navigation_flags (EphyTab *tab)
 const char *
 ephy_tab_get_status_message (EphyTab *tab)
 {
-	g_return_val_if_fail (EPHY_IS_TAB (tab), "");
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 	if (tab->priv->link_message && tab->priv->link_message[0] != '\0')
 	{
@@ -1256,7 +1271,7 @@ ephy_tab_get_status_message (EphyTab *tab)
 	}
 	else
 	{
-		return "";
+		return NULL;
 	}
 }
 
