@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: popup-commands.c,v 1.34 2003/11/22 11:32:33 marco Exp $
+ *  $Id: popup-commands.c,v 1.35 2004/01/10 10:23:57 xan Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -327,6 +327,13 @@ popup_cmd_download_link (GtkAction *action,
 			   FALSE, "link");
 }
 
+void
+popup_cmd_download_link_as (GtkAction *action,
+			    EphyWindow *window)
+{
+	save_property_url (action, _("Download link"), window, 
+			   TRUE, "link");
+}
 void
 popup_cmd_save_image_as (GtkAction *action,
 			 EphyWindow *window)
