@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: prefs-dialog.c,v 1.56 2003/12/08 17:35:32 chpe Exp $
+ *  $Id: prefs-dialog.c,v 1.64 2004/01/06 14:36:23 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,7 +34,6 @@
 #include "ephy-shell.h"
 #include "ephy-gui.h"
 #include "eel-gconf-extensions.h"
-#include "language-editor.h"
 #include "ephy-langs.h"
 #include "ephy-encodings.h"
 #include "ephy-debug.h"
@@ -77,7 +76,15 @@ void prefs_homepage_blank_button_clicked_cb	(GtkWidget *button,
 void prefs_language_more_button_clicked_cb	(GtkWidget *button,
 						 EphyDialog *dialog);
 void prefs_download_path_button_clicked_cb	(GtkWidget *button,
-						 EphyDialog *dialog);
+						 PrefsDialog *dialog);
+void language_editor_add_button_clicked_cb	(GtkWidget *button,
+						 PrefsDialog *pd);
+void language_editor_remove_button_clicked_cb	(GtkWidget *button,
+						 PrefsDialog *pd);
+void language_editor_up_button_clicked_cb	(GtkWidget *button,
+						 PrefsDialog *pd);
+void language_editor_down_button_clicked_cb	(GtkWidget *button,
+						 PrefsDialog *pd);
 
 static const
 struct
@@ -204,6 +211,8 @@ enum
 
 	/* General */
 	HOMEPAGE_ENTRY_PROP,
+	HOMEPAGE_CURRENT_PROP,
+	HOMEPAGE_BLANK_PROP,
 	AUTO_OPEN_PROP,
 	DOWNLOAD_PATH_BUTTON_PROP,
 
@@ -227,10 +236,13 @@ enum
 	/* Language */
 	AUTO_ENCODING_PROP,
 	DEFAULT_ENCODING_PROP,
-	LANGUAGE_PROP,
-	LANGUAGE_LABEL_PROP,
 	DEFAULT_ENCODING_LABEL_PROP,
-	AUTO_ENCODING_LABEL_PROP
+	AUTO_ENCODING_LABEL_PROP,
+	LANGUAGE_ADD_BUTTON_PROP,
+	LANGUAGE_REMOVE_BUTTON_PROP,
+	LANGUAGE_UP_BUTTON_PROP,
+	LANGUAGE_DOWN_BUTTON_PROP,
+	LANGUAGE_TREEVIEW_PROP
 };
 
 static const
@@ -241,6 +253,8 @@ EphyDialogProperty properties [] =
 
 	/* General */
 	{ "homepage_entry",			CONF_GENERAL_HOMEPAGE,	  PT_AUTOAPPLY,	G_TYPE_STRING },
+	{ "homepage_current_button",		NULL,			  PT_NORMAL,	0 },
+	{ "homepage_blank_button",		NULL,			  PT_NORMAL,	0 },
 	{ "auto_open_downloads_checkbutton",	CONF_AUTO_OPEN_DOWNLOADS, PT_AUTOAPPLY,	0 },
 	{ "download_path_button",		NULL,			  PT_NORMAL,	0 },
 
@@ -264,10 +278,28 @@ EphyDialogProperty properties [] =
 	/* Languages */
 	{ "auto_encoding_combo",	CONF_LANGUAGE_AUTODETECT_ENCODING,	PT_AUTOAPPLY,	G_TYPE_STRING },
 	{ "default_encoding_combo",	CONF_LANGUAGE_DEFAULT_ENCODING,		PT_AUTOAPPLY,	G_TYPE_STRING },
-	{ "language_combo",		NULL,					PT_NORMAL,	G_TYPE_STRING },
-	{ "language_label",		NULL,					PT_NORMAL,	0 },
 	{ "default_encoding_label",	NULL,					PT_NORMAL,	0 },
 	{ "auto_encoding_label",	NULL,					PT_NORMAL,	0 },
+	{ "lang_add_button",		NULL,					PT_NORMAL,	0 },
+	{ "lang_remove_button",		NULL,					PT_NORMAL,	0 },
+	{ "lang_up_button",		NULL,					PT_NORMAL,	0 },
+	{ "lang_down_button",		NULL,					PT_NORMAL,	0 },
+	{ "lang_treeview",		NULL,					PT_NORMAL,	0 },
+
+	{ NULL }
+};
+
+enum
+{
+	LANGUAGE_DIALOG,
+	LANGUAGE_PROP
+};
+
+static const
+EphyDialogProperty add_lang_props [] =
+{
+	{ "add_language_dialog", NULL,	PT_NORMAL,	0 },
+	{ "languages_treeview",	 NULL,	PT_NORMAL,	G_TYPE_STRING },
 
 	{ NULL }
 };
@@ -294,6 +326,14 @@ enum
 
 struct PrefsDialogPrivate
 {
+	GtkWidget *download_dir_chooser;
+	GtkTreeView *lang_treeview;
+	GtkTreeModel *lang_model;
+	EphyDialog *add_lang_dialog;
+	GtkWidget *lang_add_button;
+	GtkWidget *lang_remove_button;
+	GtkWidget *lang_up_button;
+	GtkWidget *lang_down_button;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -326,6 +366,29 @@ prefs_dialog_get_type (void)
 	return type;
 }
 
+static void
+prefs_dialog_finalize (GObject *object)
+{
+	PrefsDialog *dialog = EPHY_PREFS_DIALOG (object);
+
+	if (dialog->priv->add_lang_dialog != NULL)
+	{
+		g_object_remove_weak_pointer
+			(G_OBJECT (dialog->priv->add_lang_dialog),
+			(gpointer *) &dialog->priv->add_lang_dialog);
+		g_object_unref (dialog->priv->add_lang_dialog);
+	}
+
+	if (dialog->priv->download_dir_chooser != NULL)
+	{
+		g_object_remove_weak_pointer
+			(G_OBJECT (dialog->priv->download_dir_chooser),
+			 (gpointer *) &dialog->priv->download_dir_chooser);
+	}
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 prefs_dialog_class_init (PrefsDialogClass *klass)
 {
@@ -333,6 +396,8 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	object_class->finalize = prefs_dialog_finalize;
+
 	g_type_class_add_private (object_class, sizeof(PrefsDialogPrivate));
 }
 
@@ -342,14 +407,11 @@ prefs_dialog_show_help (EphyDialog *dialog)
 	GtkWidget *window, *notebook;
 	int id;
 
-	/* FIXME: Once we actually have documentation we
-	 * should point these at the correct links.
-	 */
 	char *help_preferences[] = {
-		"setting-preferences",
-		"setting-preferences",
-		"setting-preferences",
-		"setting-preferences"
+		"general-preferences",
+		"fonts-and-colors-preferences",
+		"privacy-preferences",
+		"language-preferences"
 	};
 
 	window = ephy_dialog_get_control (dialog, properties[WINDOW_PROP].id);
@@ -376,8 +438,7 @@ setup_font_combo (EphyDialog *dialog,
 	char key[255];
 	EphyEmbedSingle *single;
 
-	single = ephy_embed_shell_get_embed_single
-		(EPHY_EMBED_SHELL (ephy_shell));
+	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
 	fonts = ephy_embed_single_get_font_list (single, code);
 
 	g_snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT, type, code);
@@ -529,7 +590,7 @@ create_node_combo (EphyDialog *dialog,
 	int title_col, data_col;
 
 	code = eel_gconf_get_string (key);
-	if (code == NULL || ephy_encodings_get_node (encodings, code) == NULL)
+	if (code == NULL || ephy_encodings_get_node (encodings, code, FALSE) == NULL)
 	{
 		/* safe default */
 		eel_gconf_set_string (key, default_value);
@@ -565,60 +626,176 @@ create_node_combo (EphyDialog *dialog,
 }
 
 static void
-language_combo_changed_cb (GtkComboBox *combo,
-			   EphyDialog *dialog)
+language_editor_add (PrefsDialog *pd,
+		     const char *code,
+		     const char *desc)
 {
-	GValue value = { 0, };
-	GSList *list;
+	GtkTreeIter iter;
 
-	list = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
-	/* delete the first list item */
-	list = g_slist_remove_link (list, list);
+	g_return_if_fail (code != NULL && desc != NULL);
 
-	ephy_dialog_get_value (dialog, properties[LANGUAGE_PROP].id, &value);
+	if (gtk_tree_model_get_iter_first (pd->priv->lang_model, &iter))
+	{
+		do
+		{
+			char *c;
 
-	/* add the new language code upfront */
-	list = g_slist_prepend (list, g_value_dup_string (&value));
-	g_value_unset (&value);
+			gtk_tree_model_get (pd->priv->lang_model, &iter,
+					    COL_LANG_CODE, &c,
+					    -1);
 
-	eel_gconf_set_string_list (CONF_RENDERING_LANGUAGE, list);
+			if (strcmp (code, c) == 0)
+			{
+				g_free (c);
 
-	g_slist_foreach (list, (GFunc) g_free, NULL);
-	g_slist_free (list);
+				/* already in list, don't allow a duplicate */
+				return;
+			}
+			g_free (c);
+		}
+		while (gtk_tree_model_iter_next (pd->priv->lang_model, &iter));
+	}
+
+	gtk_list_store_append (GTK_LIST_STORE (pd->priv->lang_model), &iter);
+	
+	gtk_list_store_set (GTK_LIST_STORE (pd->priv->lang_model), &iter,
+			    COL_LANG_NAME, desc,
+			    COL_LANG_CODE, code,
+			    -1);
 }
 
 static void
-create_language_menu (EphyDialog *dialog)
+language_editor_update_pref (PrefsDialog *pd)
 {
-	GtkComboBox *combo;
+	GtkTreeIter iter;
+	GSList *codes = NULL;
+
+	if (gtk_tree_model_get_iter_first (pd->priv->lang_model, &iter))
+	{
+		do
+		{
+			char *code;
+		
+			gtk_tree_model_get (pd->priv->lang_model, &iter,
+					    COL_LANG_CODE, &code,
+					    -1);
+
+			codes = g_slist_prepend (codes, code);
+		}
+		while (gtk_tree_model_iter_next (pd->priv->lang_model, &iter));
+	}
+
+	codes = g_slist_reverse (codes);
+	eel_gconf_set_string_list (CONF_RENDERING_LANGUAGE, codes);
+
+	g_slist_foreach (codes, (GFunc) g_free, NULL);
+	g_slist_free (codes);
+}
+
+static void
+language_editor_update_buttons (PrefsDialog *dialog)
+{
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	int selected;
+
+	selection = gtk_tree_view_get_selection (dialog->priv->lang_treeview);
+
+	gtk_widget_set_sensitive (dialog->priv->lang_remove_button, FALSE);
+	gtk_widget_set_sensitive (dialog->priv->lang_up_button, FALSE);
+	gtk_widget_set_sensitive (dialog->priv->lang_down_button, FALSE);
+	
+	if (gtk_tree_selection_get_selected (selection, &model, &iter))
+	{
+		path = gtk_tree_model_get_path (model, &iter);
+	
+		selected = gtk_tree_path_get_indices (path)[0];
+	
+		gtk_widget_set_sensitive (dialog->priv->lang_remove_button, TRUE);
+		gtk_widget_set_sensitive (dialog->priv->lang_up_button, selected > 0);
+		gtk_widget_set_sensitive (dialog->priv->lang_down_button,
+			selected < gtk_tree_model_iter_n_children (model, NULL) - 1);
+
+		gtk_tree_path_free (path);
+	}
+}
+
+static void
+add_lang_dialog_response_cb (GtkWidget *widget,
+			     int response,
+			     PrefsDialog *pd)
+{
+	EphyDialog *dialog = pd->priv->add_lang_dialog;
+	GtkTreeView *treeview;
+	GtkTreeModel *model;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	GList *rows, *r;
+
+	g_return_if_fail (dialog != NULL);
+
+	if (response == GTK_RESPONSE_OK)
+	{
+		treeview = GTK_TREE_VIEW (ephy_dialog_get_control
+				(dialog, add_lang_props[LANGUAGE_PROP].id));
+		selection = gtk_tree_view_get_selection (treeview);
+
+		rows = gtk_tree_selection_get_selected_rows (selection, &model);
+
+		for (r = rows; r != NULL; r = r->next)
+		{
+			GtkTreePath *path = (GtkTreePath *) r->data;
+
+			if (gtk_tree_model_get_iter (model, &iter, path))
+			{
+				char *code, *desc;
+				
+				gtk_tree_model_get (model, &iter,
+						    COL_LANG_NAME, &desc,
+						    COL_LANG_CODE, &code,
+						    -1);
+
+				language_editor_add (pd, code, desc);
+		
+				g_free (desc);
+				g_free (code);
+			}
+		}			
+
+		g_list_foreach (rows, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free (rows);
+
+		language_editor_update_pref (pd);
+		language_editor_update_buttons (pd);
+	}
+
+	g_object_unref (dialog);
+}
+
+static EphyDialog *
+setup_add_language_dialog (PrefsDialog *pd)
+{
+	EphyDialog *dialog;
+	GtkWidget *window;
 	GtkListStore *store;
 	GtkTreeModel *sortmodel;
-	GtkTreeIter iter;
+	GtkTreeView *treeview;
 	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
 	int i;
-	GSList *list, *l, *ulist = NULL;
 
-	/* init value from first element of the list */
-	list = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
+	window = ephy_dialog_get_control (EPHY_DIALOG (pd), properties[WINDOW_PROP].id);
 
-	/* uniquify list */
-	for (l = list; l != NULL; l = l->next)
-	{
-		if (g_slist_find_custom (ulist, l->data, (GCompareFunc) strcmp) == NULL)
-		{
-			ulist = g_slist_prepend (ulist, l->data);
-		}
-		else
-		{
-			g_free (l->data);
-		}
-	}
-	g_slist_free (list);
-	list = g_slist_reverse (ulist);
-	eel_gconf_set_string_list (CONF_RENDERING_LANGUAGE, list);
+	dialog =  EPHY_DIALOG (g_object_new (EPHY_TYPE_DIALOG,
+					     "parent-window", window,
+					     NULL));
 
-	combo = GTK_COMBO_BOX (ephy_dialog_get_control
-			(dialog, properties[LANGUAGE_PROP].id));
+	ephy_dialog_construct (dialog,  add_lang_props,
+			       "prefs-dialog.glade", "add_language_dialog");
 
 	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 
@@ -632,14 +809,235 @@ create_language_menu (EphyDialog *dialog)
 				    -1);
 	}
 
-	/* add additional list items */
+	sortmodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (store));
+	gtk_tree_sortable_set_sort_column_id
+		(GTK_TREE_SORTABLE (sortmodel), COL_LANG_NAME, GTK_SORT_ASCENDING);
+
+	treeview = GTK_TREE_VIEW (ephy_dialog_get_control
+			(dialog, add_lang_props[LANGUAGE_PROP].id));
+
+	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview), FALSE);
+
+	gtk_tree_view_set_model (treeview, sortmodel);
+
+	gtk_tree_view_set_headers_visible (treeview, FALSE);
+
+	renderer = gtk_cell_renderer_text_new ();
+
+	gtk_tree_view_insert_column_with_attributes (treeview,
+						     0, "Language",
+						     renderer,
+						     "text", 0,
+						     NULL);
+	column = gtk_tree_view_get_column (treeview, 0);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_LANG_NAME);
+
+	selection = gtk_tree_view_get_selection (treeview);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
+
+	window = ephy_dialog_get_control (dialog, add_lang_props[LANGUAGE_DIALOG].id);
+	g_signal_connect (window, "response",
+			  G_CALLBACK (add_lang_dialog_response_cb), pd);
+
+	g_object_unref (store);
+	g_object_unref (sortmodel);
+
+	return dialog;
+}
+
+void
+language_editor_add_button_clicked_cb (GtkWidget *button,
+				       PrefsDialog *pd)
+{
+	if (pd->priv->add_lang_dialog == NULL)
+	{
+		pd->priv->add_lang_dialog = setup_add_language_dialog (pd);
+
+		g_object_add_weak_pointer
+			(G_OBJECT (pd->priv->add_lang_dialog),
+			(gpointer *) &pd->priv->add_lang_dialog);;
+	}
+
+	ephy_dialog_show (pd->priv->add_lang_dialog);
+}
+
+void
+language_editor_remove_button_clicked_cb (GtkWidget *button,
+					  PrefsDialog *pd)
+{
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	selection = gtk_tree_view_get_selection (pd->priv->lang_treeview);
+
+	if (gtk_tree_selection_get_selected (selection, &model, &iter))
+	{
+		gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+	}
+
+	language_editor_update_pref (pd);
+	language_editor_update_buttons (pd);
+}
+
+void
+language_editor_up_button_clicked_cb (GtkWidget *button,
+				      PrefsDialog *pd)
+{
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter, iter_prev;
+	GtkTreePath *path;
+
+	selection = gtk_tree_view_get_selection (pd->priv->lang_treeview);
+
+	if (gtk_tree_selection_get_selected (selection, &model, &iter))
+	{
+		path = gtk_tree_model_get_path (model, &iter);
+
+		if (!gtk_tree_path_prev (path))
+		{
+			gtk_tree_path_free (path);
+			return;
+		}
+
+		gtk_tree_model_get_iter (model, &iter_prev, path);
+
+		gtk_list_store_swap (GTK_LIST_STORE (model), &iter_prev, &iter);
+
+		gtk_tree_path_free (path);
+	}
+
+	language_editor_update_pref (pd);
+	language_editor_update_buttons (pd);
+}
+
+void
+language_editor_down_button_clicked_cb (GtkWidget *button,
+					PrefsDialog *pd)
+{
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter, iter_next;
+	GtkTreePath *path;
+
+
+	selection = gtk_tree_view_get_selection (pd->priv->lang_treeview);
+
+	if (gtk_tree_selection_get_selected (selection, &model, &iter))
+	{
+		path = gtk_tree_model_get_path (model, &iter);
+
+		gtk_tree_path_next (path);
+
+		gtk_tree_model_get_iter (model, &iter_next, path);
+
+		gtk_list_store_swap (GTK_LIST_STORE (model), &iter, &iter_next);
+
+		gtk_tree_path_free (path);
+	}
+
+	language_editor_update_pref (pd);
+	language_editor_update_buttons (pd);
+}
+
+static void
+language_editor_treeview_drag_end_cb (GtkWidget *widget,
+				      GdkDragContext *context,
+				      PrefsDialog *dialog)
+{
+	language_editor_update_pref (dialog);
+	language_editor_update_buttons (dialog);
+}
+
+static void
+language_editor_selection_changed_cb (GtkTreeSelection *selection,
+				      PrefsDialog *dialog)
+{
+	language_editor_update_buttons (dialog);
+}
+
+static void
+create_language_section (EphyDialog *dialog)
+{
+	PrefsDialog *pd = EPHY_PREFS_DIALOG (dialog);
+	GtkListStore *store;
+	GtkTreeView *treeview;
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+	GtkTreeSelection *selection;
+	GSList *list, *l, *ulist = NULL;
+
+	/* setup the languages treeview */
+	treeview = GTK_TREE_VIEW (ephy_dialog_get_control
+			(dialog, properties[LANGUAGE_TREEVIEW_PROP].id));
+	pd->priv->lang_treeview = treeview;
+
+	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview), TRUE);
+
+	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+
+	pd->priv->lang_model = GTK_TREE_MODEL (store);
+
+	gtk_tree_view_set_model (treeview, pd->priv->lang_model);
+	g_object_unref (store);
+
+	gtk_tree_view_set_headers_visible (treeview, FALSE);
+
+	renderer = gtk_cell_renderer_text_new ();
+
+	gtk_tree_view_insert_column_with_attributes (treeview,
+						     0, "Language",
+						     renderer,
+						     "text", 0,
+						     NULL);
+	column = gtk_tree_view_get_column (treeview, 0);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_LANG_NAME);
+
+	selection = gtk_tree_view_get_selection (treeview);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+
+	/* Connect treeview signals */
+	g_signal_connect (G_OBJECT (treeview), "drag_end",
+			  G_CALLBACK (language_editor_treeview_drag_end_cb), pd);
+	g_signal_connect (G_OBJECT (selection), "changed",
+			  G_CALLBACK (language_editor_selection_changed_cb), pd);
+
+	pd->priv->lang_add_button = ephy_dialog_get_control
+		(dialog, properties[LANGUAGE_ADD_BUTTON_PROP].id);
+	pd->priv->lang_remove_button = ephy_dialog_get_control
+		(dialog, properties[LANGUAGE_REMOVE_BUTTON_PROP].id);
+	pd->priv->lang_up_button = ephy_dialog_get_control
+		(dialog, properties[LANGUAGE_UP_BUTTON_PROP].id);
+	pd->priv->lang_down_button = ephy_dialog_get_control
+		(dialog, properties[LANGUAGE_DOWN_BUTTON_PROP].id);
+
+	list = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
+
+	/* uniquify list */
 	for (l = list; l != NULL; l = l->next)
+	{
+		if (g_slist_find_custom (ulist, l->data, (GCompareFunc) strcmp) == NULL)
+		{
+			ulist = g_slist_prepend (ulist, g_strdup (l->data));
+		}
+	}
+	ulist = g_slist_reverse (ulist);
+
+	/* if modified, write back */
+	if (g_slist_length (ulist) != g_slist_length (list))
+	{
+		eel_gconf_set_string_list (CONF_RENDERING_LANGUAGE, ulist);
+	}
+
+	/* Fill languages editor */
+	for (l = ulist; l != NULL; l = l->next)
 	{
 		const char *code = (const char *) l->data;
 		int i;
 
-		if (code == NULL) continue;
-
 		for (i = 0; i < n_languages; i++)
 		{
 			if (strcmp (languages[i].code, code) == 0) break;
@@ -651,72 +1049,67 @@ create_language_menu (EphyDialog *dialog)
 			char *text;
 
 			text = g_strdup_printf (_("Custom [%s]"), code);
-			gtk_list_store_append (store, &iter);
-			gtk_list_store_set (store, &iter,
-					    COL_LANG_NAME, text,
-					    COL_LANG_CODE, code,
-					    -1);
+			language_editor_add (pd, code, text);
 			g_free (text);
 		}
+		else
+		{
+			language_editor_add (pd, code, _(languages[i].name));
+		}
 	}
 
-	sortmodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (store));
-	gtk_tree_sortable_set_sort_column_id
-		(GTK_TREE_SORTABLE (sortmodel), COL_LANG_NAME, GTK_SORT_ASCENDING);
+	language_editor_update_buttons (pd);
 
-	gtk_combo_box_set_model (combo, sortmodel);
-
-	ephy_dialog_set_data_column
-		(dialog, properties[LANGUAGE_PROP].id, COL_LANG_CODE);
-
-        renderer = gtk_cell_renderer_text_new ();
-        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
-        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
-                                        "text", COL_LANG_NAME,
-                                        NULL);
-
-	g_signal_connect (combo, "changed",
-			  G_CALLBACK (language_combo_changed_cb), dialog);
-
-	/* set combo from first element of the list */
-	if (list && list->data)
+	/* Lockdown if key is not writable */
+	if (eel_gconf_key_is_writable (CONF_RENDERING_LANGUAGE) == FALSE)
 	{
-		const char *code = (const char *) list->data;
-		GValue value = { 0, };
-
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, code);
-		ephy_dialog_set_value (dialog, properties[LANGUAGE_PROP].id, &value);
-		g_value_unset (&value);
+		gtk_widget_set_sensitive (pd->priv->lang_add_button, FALSE);
+		gtk_widget_set_sensitive (pd->priv->lang_remove_button, FALSE);
+		gtk_widget_set_sensitive (pd->priv->lang_up_button, FALSE);
+		gtk_widget_set_sensitive (pd->priv->lang_down_button, FALSE);
+		gtk_widget_set_sensitive (GTK_WIDGET (pd->priv->lang_treeview), FALSE);
 	}
 
 	g_slist_foreach (list, (GFunc) g_free, NULL);
 	g_slist_free (list);
 
-	g_object_unref (store);
-	g_object_unref (sortmodel);
+	g_slist_foreach (ulist, (GFunc) g_free, NULL);
+	g_slist_free (ulist);
 }
 
 static char*
 get_download_button_label ()
 {
-	char *label, *key, *desktop_path, *tmp;
+	char *label, *key, *desktop_path, *home_path, *tmp;
 
 	key = eel_gconf_get_string (CONF_STATE_DOWNLOAD_DIR);
+
 	tmp = g_build_filename (g_get_home_dir (), "Desktop", NULL);
 	desktop_path = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
+	home_path = g_filename_to_utf8 (g_get_home_dir (), -1, NULL, NULL, NULL);
 	g_free (tmp);
+
 	if (g_utf8_collate (key, desktop_path) == 0)
 	{
 		g_free (key);
 		label = g_strdup (_("Desktop")); 
 	}
+	else if (g_utf8_collate (key, home_path) == 0)
+	{
+		g_free (key);
+		/* Note that this does NOT refer to the home page but to a
+		 * user's home folder. It should be translated by the same
+		 * term as GTK+'s "Home" string to be consistent with the
+		 * filechooser */
+		label = g_strdup (_("Home"));
+	}
 	else
 	{
 		label = key;
 	}
 
 	g_free (desktop_path);
+	g_free (home_path);
 	return label;
 }
 	
@@ -735,6 +1128,8 @@ create_download_path_label (EphyDialog *dialog)
 	gtk_container_add (GTK_CONTAINER (button), label);
 	g_free (dir);
 	gtk_widget_show (label);
+
+	gtk_widget_set_sensitive (button, eel_gconf_key_is_writable (CONF_STATE_DOWNLOAD_DIR));
 }
 	
 static void
@@ -742,10 +1137,10 @@ prefs_dialog_init (PrefsDialog *pd)
 {
 	EphyDialog *dialog = EPHY_DIALOG (pd);
 	EphyEncodings *encodings;
-	GtkWidget *window;
+	GtkWidget *window, *button, *combo;
 	GdkPixbuf *icon;
 	GtkCellRenderer *renderer;
-	GtkWidget *combo;
+	gboolean sensitive;
 
 	pd->priv = EPHY_PREFS_DIALOG_GET_PRIVATE (pd);
 
@@ -758,7 +1153,6 @@ prefs_dialog_init (PrefsDialog *pd)
 			      n_cookies_accept_enum, cookies_accept_enum);
 
 	ephy_dialog_set_size_group (dialog,
-				    properties[LANGUAGE_LABEL_PROP].id,
 				    properties[DEFAULT_ENCODING_LABEL_PROP].id,
 				    properties[AUTO_ENCODING_LABEL_PROP].id,
 				    NULL);
@@ -772,6 +1166,13 @@ prefs_dialog_init (PrefsDialog *pd)
 	gtk_window_set_icon (GTK_WINDOW (window), icon);
 	g_object_unref(icon);
 
+	/* set homepage button sensitivity */
+	sensitive = eel_gconf_key_is_writable (CONF_GENERAL_HOMEPAGE);
+	button = ephy_dialog_get_control (dialog, properties[HOMEPAGE_CURRENT_PROP].id);
+	gtk_widget_set_sensitive (button, sensitive);
+	button = ephy_dialog_get_control (dialog, properties[HOMEPAGE_BLANK_PROP].id);
+	gtk_widget_set_sensitive (button, sensitive);
+
 	combo = ephy_dialog_get_control (dialog, properties[VARIABLE_PROP].id);
         renderer = gtk_cell_renderer_text_new ();
         gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
@@ -799,7 +1200,7 @@ prefs_dialog_init (PrefsDialog *pd)
 			   ephy_encodings_get_detectors (encodings),
 			   CONF_LANGUAGE_AUTODETECT_ENCODING, "");
 
-	create_language_menu (dialog);
+	create_language_section	(dialog);
 
 	create_download_path_label (dialog);
 }
@@ -825,9 +1226,7 @@ prefs_clear_cache_button_clicked_cb (GtkWidget *button,
 {
 	EphyEmbedSingle *single;
 
-	single = ephy_embed_shell_get_embed_single
-		(EPHY_EMBED_SHELL (ephy_shell));
-
+	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
 	ephy_embed_single_clear_cache (single);
 }
 
@@ -870,80 +1269,6 @@ prefs_homepage_blank_button_clicked_cb (GtkWidget *button,
 	set_homepage_entry (dialog, NULL);
 }
 
-static void
-language_dialog_changed_cb (LanguageEditor *le,
-			    GSList *list,
-			    EphyDialog *dialog)
-{
-	LOG ("language_dialog_changed_cb")
-
-	if (list && list->data)
-	{
-		GValue value = { 0, };
-
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, (char *) list->data);
-		ephy_dialog_set_value (dialog, properties[LANGUAGE_PROP].id, &value);
-		g_value_unset (&value);
-	}
-
-	eel_gconf_set_string_list (CONF_RENDERING_LANGUAGE, list);
-}
-
-void
-prefs_language_more_button_clicked_cb (GtkWidget *button,
-				       EphyDialog *dialog)
-{
-	LanguageEditor *editor;
-	GtkWidget *window, *combo;
-	GtkTreeModel *model;
-	GSList *codes, *l;
-
-	window = ephy_dialog_get_control (dialog, properties[WINDOW_PROP].id);
-	combo = ephy_dialog_get_control (dialog, properties[LANGUAGE_PROP].id);
-	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
-
-	editor = language_editor_new (window);
-	language_editor_set_model (editor, model);
-
-	codes = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
-	for (l = codes; l != NULL; l = l->next)
-	{
-		const char *code = (const char *) l->data;
-		int i;
-
-		if (code == NULL) continue;
-
-		for (i = 0; i < n_languages; i++)
-		{
-			if (strcmp (languages[i].code, code) == 0) break;
-		}
-
-		if (i == n_languages)
-		{
-			char *desc;
-			desc = g_strdup_printf (_("Custom [%s]"), code);
-			language_editor_add (editor, code, desc);\
-			g_free (desc);
-		}
-		else
-		{
-			language_editor_add (editor, code, _(languages[i].name));
-		}
-	}
-	g_slist_foreach (codes, (GFunc) g_free, NULL);
-	g_slist_free (codes);
-
-	/* FIXME: make it only modal to prefs dialogue, not to all windows */
-	ephy_dialog_set_modal (EPHY_DIALOG (editor), TRUE);
-
-	g_signal_connect (editor, "list-changed",
-			  G_CALLBACK (language_dialog_changed_cb),
-			  dialog);
-
-	ephy_dialog_show (EPHY_DIALOG (editor));
-}
-
 static void
 download_path_response_cb (GtkDialog *fc, gint response, EphyDialog *dialog)
 {
@@ -955,14 +1280,17 @@ download_path_response_cb (GtkDialog *fc, gint response, EphyDialog *dialog)
 		if (dir != NULL)
 		{
 			GtkWidget *button;
-			char *label;
+			char *label, *converted;
 
-			eel_gconf_set_string (CONF_STATE_DOWNLOAD_DIR, dir);
+			converted = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
+			eel_gconf_set_string (CONF_STATE_DOWNLOAD_DIR, converted);
 			
 			button = ephy_dialog_get_control (dialog, properties[DOWNLOAD_PATH_BUTTON_PROP].id);
 			label = get_download_button_label ();
 			ephy_ellipsizing_label_set_text ((EphyEllipsizingLabel*) GTK_BIN (button)->child,
 							  label);
+
+			g_free (converted);
 			g_free (dir);
 			g_free (label);
 		}
@@ -973,22 +1301,29 @@ download_path_response_cb (GtkDialog *fc, gint response, EphyDialog *dialog)
 
 void
 prefs_download_path_button_clicked_cb (GtkWidget *button,
-				       EphyDialog *dialog)
+				       PrefsDialog *dialog)
 {
-	GtkWidget *parent;
-	EphyFileChooser *fc;
+	if (dialog->priv->download_dir_chooser == NULL)
+	{
+		GtkWidget *parent;
+		EphyFileChooser *fc;
 
-	parent = ephy_dialog_get_control (dialog, properties[WINDOW_PROP].id);
+		parent = ephy_dialog_get_control
+			(EPHY_DIALOG (dialog), properties[WINDOW_PROP].id);
+	
+		fc = ephy_file_chooser_new (_("Select a directory"),
+					    GTK_WIDGET (parent),
+					    GTK_FILE_CHOOSER_ACTION_OPEN,
+					    NULL);
+		gtk_file_chooser_set_folder_mode (GTK_FILE_CHOOSER (fc), TRUE);
+	
+		g_signal_connect (GTK_DIALOG (fc), "response",
+				    G_CALLBACK (download_path_response_cb),
+				    dialog);
+		dialog->priv->download_dir_chooser = GTK_WIDGET (fc);
+		g_object_add_weak_pointer
+			(G_OBJECT (fc), (gpointer *) &dialog->priv->download_dir_chooser);
+	}
 
-	fc = ephy_file_chooser_new (_("Select a directory"),
-				    GTK_WIDGET (parent),
-				    GTK_FILE_CHOOSER_ACTION_OPEN,
-				    NULL);
-	gtk_file_chooser_set_folder_mode (GTK_FILE_CHOOSER (fc), TRUE);
-
-	g_signal_connect (GTK_DIALOG (fc), "response",
-			    G_CALLBACK (download_path_response_cb),
-			    dialog);
-
-	gtk_widget_show (GTK_WIDGET (fc));
+	gtk_widget_show (dialog->priv->download_dir_chooser);
 }
