@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,41 +15,47 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.76.2.5 2004/01/19 11:59:42 chpe Exp $
+ *  $Id: window-commands.c,v 1.104 2004/01/09 09:29:56 chpe Exp $
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "ephy-shell.h"
+#include "ephy-embed-factory.h"
+#include "ephy-embed-persist.h"
 #include "ephy-debug.h"
+#include "ephy-command-manager.h"
 #include "window-commands.h"
 #include "print-dialog.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
-#include "ephy-embed-utils.h"
-#include "pdm-dialog.h"
+#include "ephy-embed-prefs.h"
+#include "ephy-dialog.h"
 #include "ephy-bookmarks-editor.h"
+#include "ephy-history-window.h"
 #include "ephy-new-bookmark.h"
-#include "egg-toggle-action.h"
-#include "egg-editable-toolbar.h"
-#include "egg-toolbar-editor.h"
+#include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
 #include "toolbar.h"
 #include "ephy-state.h"
 #include "ephy-gui.h"
 #include "ephy-zoom.h"
-#include "prefs-dialog.h"
 #include "ephy-toolbars-model.h"
+#include "egg-editable-toolbar.h"
+#include "egg-toolbar-editor.h"
 
 #include <string.h>
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <bonobo/bonobo-i18n.h>
-#include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-stock-icons.h>
-#include <libgnomeui/gnome-icon-theme.h>
-#include <libgnome/gnome-program.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <gtk/gtkeditable.h>
+#include <gtk/gtktoggleaction.h>
+#include <libgnomeui/gnome-about.h>
+#include <glib/gi18n.h>
 
 enum
 {
@@ -57,21 +63,50 @@ enum
 };
 
 void
-window_cmd_edit_find (EggAction *action,
+window_cmd_edit_find (GtkAction *action,
 		      EphyWindow *window)
 {
 	ephy_window_find (window);
 }
 
 void
-window_cmd_file_print (EggAction *action,
+window_cmd_file_print_setup (GtkAction *action,
+			     EphyWindow *window)
+{
+	EphyDialog *dialog;
+
+	dialog = EPHY_DIALOG (ephy_shell_get_print_setup_dialog (ephy_shell));
+
+	ephy_dialog_show (dialog);
+}
+
+void
+window_cmd_file_print_preview (GtkAction *action,
+			       EphyWindow *window)
+{
+	EphyEmbed *embed;
+	EmbedPrintInfo *info;
+
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (EPHY_IS_EMBED (embed));
+
+	info = ephy_print_get_print_info ();
+	info->preview = TRUE;
+
+	ephy_embed_print (embed, info);
+
+	ephy_window_set_print_preview (window, TRUE);
+}
+
+void
+window_cmd_file_print (GtkAction *action,
 		       EphyWindow *window)
 {
 	ephy_window_print (window);
 }
 
 void
-window_cmd_go_back (EggAction *action,
+window_cmd_go_back (GtkAction *action,
 		    EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -85,7 +120,7 @@ window_cmd_go_back (EggAction *action,
 }
 
 void
-window_cmd_go_up (EggAction *action,
+window_cmd_go_up (GtkAction *action,
 		  EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -99,7 +134,7 @@ window_cmd_go_up (EggAction *action,
 }
 
 void
-window_cmd_file_send_to	(EggAction *action,
+window_cmd_file_send_to	(GtkAction *action,
 			 EphyWindow *window)
 {
 	EphyTab *tab;
@@ -115,11 +150,12 @@ window_cmd_file_send_to	(EggAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_get_location (embed, TRUE, &embed_location);
+	embed_location = ephy_embed_get_location (embed, TRUE);
 	location = gnome_vfs_escape_string (embed_location);
 	g_free (embed_location);
 
-	if (ephy_embed_get_title (embed, &title) == G_OK)
+	title = ephy_embed_get_title (embed);
+	if (title != NULL)
 	{
 		char *tmp = gnome_vfs_escape_string (title);
 		g_free (title);
@@ -142,7 +178,7 @@ window_cmd_file_send_to	(EggAction *action,
 }
 
 void
-window_cmd_go_forward (EggAction *action,
+window_cmd_go_forward (GtkAction *action,
 		       EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -156,7 +192,7 @@ window_cmd_go_forward (EggAction *action,
 }
 
 void
-window_cmd_go_home (EggAction *action,
+window_cmd_go_home (GtkAction *action,
 		    EphyWindow *window)
 {
 	char *location;
@@ -176,14 +212,14 @@ window_cmd_go_home (EggAction *action,
 }
 
 void
-window_cmd_go_location (EggAction *action,
+window_cmd_go_location (GtkAction *action,
 		        EphyWindow *window)
 {
 	ephy_window_activate_location (window);
 }
 
 void
-window_cmd_view_stop (EggAction *action,
+window_cmd_view_stop (GtkAction *action,
 		      EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -197,7 +233,7 @@ window_cmd_view_stop (EggAction *action,
 }
 
 void
-window_cmd_view_reload (EggAction *action,
+window_cmd_view_reload (GtkAction *action,
 		        EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -238,7 +274,7 @@ window_cmd_view_reload (EggAction *action,
 }
 
 void
-window_cmd_file_new_window (EggAction *action,
+window_cmd_file_new_window (GtkAction *action,
 		            EphyWindow *window)
 {
 	EphyTab *tab;
@@ -252,7 +288,7 @@ window_cmd_file_new_window (EggAction *action,
 }
 
 void
-window_cmd_file_new_tab (EggAction *action,
+window_cmd_file_new_tab (GtkAction *action,
 		         EphyWindow *window)
 {
 	EphyTab *tab;
@@ -267,14 +303,19 @@ window_cmd_file_new_tab (EggAction *action,
 }
 
 void
-window_cmd_go_bookmarks (EggAction *action,
+window_cmd_go_bookmarks (GtkAction *action,
 			 EphyWindow *window)
 {
-	ephy_shell_show_bookmarks_editor (ephy_shell, GTK_WIDGET (window));
+	GtkWidget *bwindow;
+
+	bwindow = ephy_shell_get_bookmarks_editor (ephy_shell);
+	ephy_bookmarks_editor_set_parent (EPHY_BOOKMARKS_EDITOR (bwindow),
+			                  GTK_WIDGET (window));
+	gtk_window_present (GTK_WINDOW (bwindow));
 }
 
 void
-window_cmd_file_bookmark_page (EggAction *action,
+window_cmd_file_bookmark_page (GtkAction *action,
 			       EphyWindow *window)
 {
 	EphyTab *tab;
@@ -291,9 +332,10 @@ window_cmd_file_bookmark_page (EggAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_get_location (embed, TRUE, &location);
+	location = ephy_embed_get_location (embed, TRUE);
 
-	if (ephy_embed_get_title (embed, &title) != G_OK)
+	title = ephy_embed_get_title (embed);
+	if (title == NULL)
 	{
 		title = g_strdup (_("Untitled"));
 	}
@@ -317,57 +359,47 @@ window_cmd_file_bookmark_page (EggAction *action,
 	g_free (location);
 }
 
-void
-window_cmd_file_open (EggAction *action,
-		      EphyWindow *window)
+static void
+open_response_cb (GtkDialog *dialog, gint response, EphyWindow *window)
 {
-	char *dir, *ret_dir, *file;
-	EphyEmbedShell *embed_shell;
-	gresult result;
-	EphyEmbedSingle *single;
-
-	single = ephy_embed_shell_get_embed_single
-		(EPHY_EMBED_SHELL (ephy_shell));
-
-	embed_shell = EPHY_EMBED_SHELL (ephy_shell);
-
-        dir = eel_gconf_get_string (CONF_STATE_OPEN_DIR);
-
-	result = ephy_embed_single_show_file_picker
-		(single, GTK_WIDGET (window),
-		 _("Open"),
-                 dir, NULL, modeOpen,
-                 &file, NULL, NULL);
-
-	/* persist directory choice */
-	/* Fix for bug 122780:
-	 * if the user selected a directory, or aborted with no filename typed,
-	 * g_path_get_dirname and gnome_vfs_uri_extract_dirname strip the last
-	 * path component, so test if the returned file is actually a directory.
-	 */
-	if (g_file_test (file, G_FILE_TEST_IS_DIR))
+	if (response == EPHY_RESPONSE_OPEN)
 	{
-		ret_dir = g_strdup (file);
-	}
-	else
-	{
-		ret_dir = g_path_get_dirname (file);
-	}
+		char *uri, *converted;
 
-	eel_gconf_set_string (CONF_STATE_OPEN_DIR, ret_dir);
+		uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+		converted = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
 
-	if (result == G_OK)
-	{
-		ephy_window_load_url(window, file);
+		if (converted != NULL)
+		{
+			ephy_window_load_url(window, converted);
+		}
+
+		g_free (converted);
+		g_free (uri);
         }
 
-	g_free (ret_dir);
-	g_free (file);
-        g_free (dir);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 void
-window_cmd_file_save_as (EggAction *action,
+window_cmd_file_open (GtkAction *action,
+		      EphyWindow *window)
+{
+	EphyFileChooser *dialog;
+
+	dialog = ephy_file_chooser_new (_("Open"),
+					GTK_WIDGET (window),
+					GTK_FILE_CHOOSER_ACTION_OPEN,
+					CONF_STATE_OPEN_DIR);
+
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (open_response_cb), window);
+
+	gtk_widget_show (GTK_WIDGET (dialog));
+}
+
+void
+window_cmd_file_save_as (GtkAction *action,
 			 EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -376,22 +408,24 @@ window_cmd_file_save_as (EggAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	persist = ephy_embed_persist_new (embed);
-	ephy_embed_persist_set_flags (persist,
-				      EMBED_PERSIST_MAINDOC);
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
 
-	ephy_embed_utils_save (GTK_WIDGET(window),
-			       _("Save As"),
-			       CONF_STATE_SAVE_DIR,
-			       TRUE,
-			       TRUE,
-			       persist);
+	ephy_embed_persist_set_embed (persist, embed);
+	ephy_embed_persist_set_fc_title (persist, _("Save As"));
+	ephy_embed_persist_set_fc_parent (persist,GTK_WINDOW (window));
+	ephy_embed_persist_set_flags
+		(persist, EMBED_PERSIST_MAINDOC | EMBED_PERSIST_ASK_DESTINATION);
+	ephy_embed_persist_set_persist_key
+		(persist, CONF_STATE_SAVE_DIR);
+
+	ephy_embed_persist_save (persist);
 
 	g_object_unref (G_OBJECT(persist));
 }
 
 void
-window_cmd_file_close_window (EggAction *action,
+window_cmd_file_close_window (GtkAction *action,
 		              EphyWindow *window)
 {
 	EphyTab *tab;
@@ -403,7 +437,41 @@ window_cmd_file_close_window (EggAction *action,
 }
 
 void
-window_cmd_edit_cut (EggAction *action,
+window_cmd_edit_undo (GtkAction *action,
+		      EphyWindow *window)
+{
+	GtkWidget *widget;
+	GtkWidget *embed;
+
+	widget = gtk_window_get_focus (GTK_WINDOW (window));
+	embed = gtk_widget_get_ancestor (widget, EPHY_TYPE_EMBED);
+
+	if (embed)
+	{
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed), 
+						 "cmd_undo");
+	}
+}
+
+void
+window_cmd_edit_redo (GtkAction *action,
+		      EphyWindow *window)
+{
+	GtkWidget *widget;
+	GtkWidget *embed;
+
+	widget = gtk_window_get_focus (GTK_WINDOW (window));
+	embed = gtk_widget_get_ancestor (widget, EPHY_TYPE_EMBED);
+
+	if (embed)
+	{
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed), 
+						 "cmd_redo");
+	}
+}
+
+void
+window_cmd_edit_cut (GtkAction *action,
 		     EphyWindow *window)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
@@ -418,12 +486,13 @@ window_cmd_edit_cut (EggAction *action,
 		embed = ephy_window_get_active_embed (window);
 		g_return_if_fail (embed != NULL);
 
-		ephy_embed_selection_cut (embed);
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
+						 "cmd_cut");
 	}
 }
 
 void
-window_cmd_edit_copy (EggAction *action,
+window_cmd_edit_copy (GtkAction *action,
 		      EphyWindow *window)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
@@ -439,12 +508,13 @@ window_cmd_edit_copy (EggAction *action,
 		embed = ephy_window_get_active_embed (window);
 		g_return_if_fail (embed != NULL);
 
-		ephy_embed_selection_copy (embed);
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
+						 "cmd_copy");
 	}
 }
 
 void
-window_cmd_edit_paste (EggAction *action,
+window_cmd_edit_paste (GtkAction *action,
 		       EphyWindow *window)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
@@ -460,12 +530,13 @@ window_cmd_edit_paste (EggAction *action,
 		embed = ephy_window_get_active_embed (window);
 		g_return_if_fail (embed != NULL);
 
-		ephy_embed_paste (embed);
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
+						 "cmd_paste");
 	}
 }
 
 void
-window_cmd_edit_select_all (EggAction *action,
+window_cmd_edit_select_all (GtkAction *action,
 			    EphyWindow *window)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
@@ -481,12 +552,13 @@ window_cmd_edit_select_all (EggAction *action,
 		embed = ephy_window_get_active_embed (window);
 		g_return_if_fail (embed != NULL);
 
-		ephy_embed_select_all (embed);
+		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
+						 "cmd_selectAll");
 	}
 }
 
 void
-window_cmd_edit_find_next (EggAction *action,
+window_cmd_edit_find_next (GtkAction *action,
 			   EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -498,7 +570,7 @@ window_cmd_edit_find_next (EggAction *action,
 }
 
 void
-window_cmd_edit_find_prev (EggAction *action,
+window_cmd_edit_find_prev (GtkAction *action,
 			   EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -510,67 +582,40 @@ window_cmd_edit_find_prev (EggAction *action,
 }
 
 void
-window_cmd_view_bookmarks_bar (EggAction *action,
+window_cmd_view_bookmarks_bar (GtkAction *action,
 			       EphyWindow *window)
 {
-	EmbedChromeMask mask;
 	gboolean active;
-	gboolean current_state;
 
-	mask = ephy_window_get_chrome (window);
-	active = EGG_TOGGLE_ACTION (action)->active;
-	current_state = (mask & EMBED_CHROME_BOOKMARKSBARON) > 0;
-
-	if (active != current_state)
-	{
-		mask ^= EMBED_CHROME_BOOKMARKSBARON;
-		ephy_window_set_chrome (window, mask);
-	}
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	eel_gconf_set_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR, active);
 }
 
 void
-window_cmd_view_toolbar (EggAction *action,
+window_cmd_view_toolbar (GtkAction *action,
 			 EphyWindow *window)
 {
-	EmbedChromeMask mask;
 	gboolean active;
-	gboolean current_state;
 
-	mask = ephy_window_get_chrome (window);
-	active = EGG_TOGGLE_ACTION (action)->active;
-	current_state = (mask & EMBED_CHROME_TOOLBARON) > 0;
-
-	if (active != current_state)
-	{
-		mask ^= EMBED_CHROME_TOOLBARON;
-		ephy_window_set_chrome (window, mask);
-	}
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	eel_gconf_set_boolean (CONF_WINDOWS_SHOW_TOOLBARS, active);
 }
 
 void
-window_cmd_view_statusbar (EggAction *action,
+window_cmd_view_statusbar (GtkAction *action,
 			   EphyWindow *window)
 {
-	EmbedChromeMask mask;
 	gboolean active;
-	gboolean current_state;
 
-	mask = ephy_window_get_chrome (window);
-	active = EGG_TOGGLE_ACTION (action)->active;
-	current_state = (mask & EMBED_CHROME_STATUSBARON) > 0;
-
-	if (active != current_state)
-	{
-		mask ^= EMBED_CHROME_STATUSBARON;
-		ephy_window_set_chrome (window, mask);
-	}
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	eel_gconf_set_boolean (CONF_WINDOWS_SHOW_STATUSBAR, active);
 }
 
 void
-window_cmd_view_fullscreen (EggAction *action,
+window_cmd_view_fullscreen (GtkAction *action,
 			    EphyWindow *window)
 {
-	if (EGG_TOGGLE_ACTION (action)->active)
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
 	{
 		gtk_window_fullscreen (GTK_WINDOW (window));
 	}
@@ -581,65 +626,191 @@ window_cmd_view_fullscreen (EggAction *action,
 }
 
 void
-window_cmd_view_zoom_in	(EggAction *action,
+window_cmd_view_zoom_in	(GtkAction *action,
 			 EphyWindow *window)
 {
 	ephy_window_set_zoom (window, ZOOM_IN);
 }
 
 void
-window_cmd_view_zoom_out (EggAction *action,
+window_cmd_view_zoom_out (GtkAction *action,
 			  EphyWindow *window)
 {
 	ephy_window_set_zoom (window, ZOOM_OUT);
 }
 
 void
-window_cmd_view_zoom_normal (EggAction *action,
+window_cmd_view_zoom_normal (GtkAction *action,
 			     EphyWindow *window)
 {
 	ephy_window_set_zoom (window, 1.0);
 }
 
+static GnomeVFSMimeApplication *
+get_editor_application (void)
+{
+	GnomeVFSMimeApplication *app;
+
+	app = gnome_vfs_mime_get_default_application ("text/plain");
+	if (app == NULL)
+	{
+		g_warning ("Cannot find a text editor.");
+	}
+	return app;
+}
+
+static void
+editor_open_uri (const char *address)
+{
+	GList *uris = NULL;
+	char *canonical;
+	GnomeVFSMimeApplication *app;
+
+	canonical = gnome_vfs_make_uri_canonical (address);
+
+	uris = g_list_append (uris, canonical);
+
+	app = get_editor_application ();
+	if (app)
+	{
+		gnome_vfs_mime_application_launch (app, uris);
+		gnome_vfs_mime_application_free (app);
+	}
+
+	g_free (canonical);
+	g_list_free (uris);
+}
+
+static void
+save_source_completed_cb (EphyEmbedPersist *persist)
+{
+	const char *dest;
+
+	dest = ephy_embed_persist_get_dest (persist);
+	g_return_if_fail (dest != NULL);
+
+	ephy_shell_delete_on_exit (ephy_shell, dest);
+
+	editor_open_uri (dest);
+}
+
+static gboolean
+editor_can_open_uri (char *address)
+{
+	GnomeVFSMimeApplication *app;
+	GnomeVFSURI *uri;
+	const char *scheme;
+	gboolean result = FALSE;
+
+	app = get_editor_application ();
+
+	uri = gnome_vfs_uri_new (address);
+	scheme = uri ? gnome_vfs_uri_get_scheme (uri) : NULL;
+
+	/* Open directly only read/write protocols, otherwise
+	   you just get extra network overhead without any advantage */
+	if (scheme && strcmp (scheme, "file") != 0)
+	{
+		scheme = NULL;
+	}
+	
+	if (scheme && app && app->supported_uri_schemes)
+	{
+		if (g_list_find_custom (app->supported_uri_schemes,
+                                        scheme, (GCompareFunc) strcmp))
+		{
+			result = TRUE;
+		}
+	}
+
+	if (uri)
+	{
+		gnome_vfs_uri_unref (uri);
+	}
+
+	return result;
+}
+
+static void
+save_temp_source (EphyEmbed *embed)
+{
+	char *tmp, *base;
+	EphyEmbedPersist *persist;
+
+	base = g_build_filename (g_get_tmp_dir (), "viewsourceXXXXXX", NULL);
+	tmp = ephy_file_tmp_filename (base, "html");
+	g_free (base);
+
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+
+	ephy_embed_persist_set_embed (persist, embed);
+	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_COPY_PAGE |
+				      EMBED_PERSIST_NO_VIEW);
+	ephy_embed_persist_set_dest (persist, tmp);
+
+	g_signal_connect (persist, "completed",
+			  G_CALLBACK (save_source_completed_cb), NULL);
+
+	ephy_embed_persist_save (persist);
+	g_object_unref (persist);
+
+	g_free (tmp);
+}
+
 void
-window_cmd_view_page_source (EggAction *action,
+window_cmd_view_page_source (GtkAction *action,
 			     EphyWindow *window)
 {
-	EphyTab *tab;
+	EphyEmbed *embed;
+	char *address;
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (tab != NULL);
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
 
-	ephy_shell_new_tab (ephy_shell, window, tab, NULL,
-			    EPHY_NEW_TAB_CLONE_PAGE |
-			    EPHY_NEW_TAB_SOURCE_MODE);
+	address = ephy_embed_get_location (embed, TRUE);
+
+	if (editor_can_open_uri (address))
+	{
+		editor_open_uri (address);
+	}
+	else
+	{
+		save_temp_source (embed);
+	}
 }
 
 void
-window_cmd_go_history (EggAction *action,
+window_cmd_go_history (GtkAction *action,
 		       EphyWindow *window)
 {
-	ephy_shell_show_history_window (ephy_shell, GTK_WIDGET (window));
+	GtkWidget *hwindow;
+
+	hwindow = ephy_shell_get_history_window (ephy_shell);
+	ephy_history_window_set_parent (EPHY_HISTORY_WINDOW (hwindow),
+					GTK_WIDGET (window));
+	gtk_window_present (GTK_WINDOW (hwindow));
 }
 
 void
-window_cmd_edit_personal_data (EggAction *action,
+window_cmd_edit_personal_data (GtkAction *action,
 		               EphyWindow *window)
 {
 	EphyDialog *dialog;
 
-	dialog = pdm_dialog_new (GTK_WIDGET(window));
+	dialog = EPHY_DIALOG (ephy_shell_get_pdm_dialog (ephy_shell));
 
 	ephy_dialog_show (dialog);
 }
 
 void
-window_cmd_edit_prefs (EggAction *action,
+window_cmd_edit_prefs (GtkAction *action,
 		       EphyWindow *window)
 {
 	EphyDialog *dialog;
 
-	dialog = prefs_dialog_new (GTK_WIDGET (window));
+	dialog = EPHY_DIALOG (ephy_shell_get_prefs_dialog (ephy_shell));
+
 	ephy_dialog_show (dialog);
 }
 
@@ -665,28 +836,28 @@ toolbar_editor_response_cb (GtkDialog  *dialog,
 		break;
 	case RESPONSE_ADD_TOOLBAR:
 		model = EPHY_TOOLBARS_MODEL
-			(ephy_shell_get_toolbars_model (ephy_shell));
+			(ephy_shell_get_toolbars_model (ephy_shell, FALSE));
 		n = egg_toolbars_model_n_toolbars (EGG_TOOLBARS_MODEL (model));
 		egg_toolbars_model_add_toolbar (EGG_TOOLBARS_MODEL (model),
 						n - 1, "UserCreated");
 		break;
 	case GTK_RESPONSE_HELP:
-		/* FIXME: Connect toolbar editor help */
+		ephy_gui_help (GTK_WINDOW (dialog), "epiphany", "to-edit-toolbars");
 		break;
 	}
 }
 
 void
-window_cmd_edit_toolbar (EggAction *action,
+window_cmd_edit_toolbar (GtkAction *action,
 			 EphyWindow *window)
 {
 	GtkWidget *editor;
 	EphyToolbarsModel *model;
-	Toolbar *t;
+	GtkWidget *t;
 	GtkWidget *dialog;
 
 	model = EPHY_TOOLBARS_MODEL
-		(ephy_shell_get_toolbars_model (ephy_shell));
+		(ephy_shell_get_toolbars_model (ephy_shell, FALSE));
 	t = ephy_window_get_toolbar (window);
 
 	dialog = gtk_dialog_new ();
@@ -696,7 +867,7 @@ window_cmd_edit_toolbar (EggAction *action,
 				      GTK_WINDOW (window));
 
 	editor = egg_toolbar_editor_new
-		(EGG_MENU_MERGE (window->ui_merge),
+		(GTK_UI_MANAGER (window->ui_merge),
 		 EGG_TOOLBARS_MODEL (model));
 	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor),
 					 ephy_file ("epiphany-toolbar.xml"));
@@ -731,24 +902,25 @@ window_cmd_edit_toolbar (EggAction *action,
 }
 
 void
-window_cmd_help_contents (EggAction *action,
+window_cmd_help_contents (GtkAction *action,
 			 EphyWindow *window)
 {
 	ephy_gui_help (GTK_WINDOW (window), "epiphany", NULL);
 }
 
 void
-window_cmd_help_about (EggAction *action,
+window_cmd_help_about (GtkAction *action,
 		       GtkWidget *window)
 {
 	static GtkWidget *about = NULL;
 	GtkWidget** ptr;
 	GdkPixbuf *icon;
-	char *icon_path;
-	GdkPixbuf *logo;
-	GnomeIconTheme *icon_theme;
+	const char *icon_path;
+	GdkPixbuf *logo = NULL;
+	GtkIconTheme *icon_theme;
+	GtkIconInfo *icon_info;
 
-	static gchar *authors[] = {
+	static char *authors[] = {
 		"Marco Pesenti Gritti <marco@gnome.org>",
 		"Xan Lopez <xan@gnome.org>",
 		"David Bordoley <bordoley@msu.edu>",
@@ -756,14 +928,15 @@ window_cmd_help_about (EggAction *action,
 		NULL
 	};
 
-	gchar *documenters[] = {
+	char *documenters[] = {
 		"Patanjali Somayaji <patanjali@codito.com>",
 		"David Bordoley <bordoley@msu.edu>",
+		"Piers Cornwell", 
 		NULL
 	};
 
 	/* Translator credits */
-	gchar *translator_credits = _("translator_credits");
+	char *translator_credits = _("translator_credits");
 
 	if (about != NULL)
 	{
@@ -772,19 +945,21 @@ window_cmd_help_about (EggAction *action,
 		return;
 	}
 
-	icon_theme = gnome_icon_theme_new ();
-	icon_path = gnome_icon_theme_lookup_icon (icon_theme, "web-browser",
-						  -1, NULL, NULL);
-	g_object_unref (icon_theme);
+	/* FIXME: use the icon theme for the correct screen, not for the default screen */
+	icon_theme = gtk_icon_theme_get_default ();
+	icon_info = gtk_icon_theme_lookup_icon (icon_theme, "web-browser", -1, 0);
 
-	if (icon_path)
+	if (icon_info)
 	{
-		logo = gdk_pixbuf_new_from_file (icon_path, NULL);
-		g_free (icon_path);
+		icon_path = gtk_icon_info_get_filename (icon_info);
+
+		if (icon_path != NULL)
+		{
+			logo = gdk_pixbuf_new_from_file (icon_path, NULL);
+		}
 	}
 	else
 	{
-		logo = NULL;
 		g_warning ("Web browser gnome icon not found");
 	}
 
@@ -818,7 +993,7 @@ window_cmd_help_about (EggAction *action,
 }
 
 void
-window_cmd_tabs_next (EggAction *action,
+window_cmd_tabs_next (GtkAction *action,
 		      EphyWindow *window)
 {
 	GtkNotebook *nb;
@@ -837,7 +1012,7 @@ window_cmd_tabs_next (EggAction *action,
 }
 
 void
-window_cmd_tabs_previous (EggAction *action,
+window_cmd_tabs_previous (GtkAction *action,
 			  EphyWindow *window)
 {
 	GtkNotebook *nb;
@@ -856,7 +1031,7 @@ window_cmd_tabs_previous (EggAction *action,
 }
 
 void
-window_cmd_tabs_move_left  (EggAction *action,
+window_cmd_tabs_move_left  (GtkAction *action,
 			    EphyWindow *window)
 {
 	GtkWidget *notebook;
@@ -874,7 +1049,7 @@ window_cmd_tabs_move_left  (EggAction *action,
 	}
 }
 
-void window_cmd_tabs_move_right (EggAction *action,
+void window_cmd_tabs_move_right (GtkAction *action,
 				 EphyWindow *window)
 {
 	GtkWidget *notebook;
@@ -895,7 +1070,7 @@ void window_cmd_tabs_move_right (EggAction *action,
 }
 
 void
-window_cmd_tabs_detach  (EggAction *action,
+window_cmd_tabs_detach  (GtkAction *action,
 			 EphyWindow *window)
 {
 	EphyTab *tab;
@@ -916,19 +1091,27 @@ window_cmd_tabs_detach  (EggAction *action,
 }
 
 void
-window_cmd_load_location (EggAction *action,
+window_cmd_load_location (GtkAction *action,
 			  EphyWindow *window)
 {
 	Toolbar *toolbar;
-	char *location;
+	const char *location;
 
-	toolbar = ephy_window_get_toolbar (window);
+	toolbar = EPHY_TOOLBAR (ephy_window_get_toolbar (window));
 	location = toolbar_get_location (toolbar);
 
 	if (location)
 	{
 		ephy_window_load_url (window, location);
-		g_free (location);
 	}
 }
 
+void
+window_cmd_browse_with_caret (GtkAction *action,
+			      EphyWindow *window)
+{
+	gboolean active;
+
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+	eel_gconf_set_boolean (CONF_BROWSE_WITH_CARET, active);
+}
